
//  ASL IPC MANAGER LIBRARY 
//
//  ASL (Abstract Schema Language) System Comunication Kernel
//
//
//
//  FILE NAME   : aslcom_seq.c
//  MODULE TYPE : ASL Comunication System File
//  STATUS      : In development 
//  PURPOSE     : Comunication between processes
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyright 1995  
//
//
//
//  HISTORY : Version (Date-Name) Modification, description.  
//
//            1.0 (Oct 28, 1995 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//            1.1 (Nov 11, 1997 - Oscar Peguero), 
//            Addition of members in AslIPCManager class.
//
//            1.2 (Feb 4, 1998 - Oscar Peguero), 
//            Simple version for PC with double buffering (no threads).

#include "nsl_all_kernel_include.h"  
#include "asl_all_rts_include.h"  

// AslBuffer methods
// AslIPCManager methods

AslIPCManager::AslIPCManager(void)
{
	AslIPCInitialize(0);
}

AslIPCManager::~AslIPCManager(void)
{	
	delete ComMutex;
}

void AslIPCManager::AslIPCInitialize(int port)
{	
	if (asl_debug)
		aslio<<"ASL IPC MANAGER: Creating session layer at "<<0
		     <<" port"<<aslend;
	
	AslCom	  = this;
	msgCount  = 1;
	bidCount  = 1;
	INTERRUPT = 0;
	
	AslGetEnvironment();    
	 
	systemType = ASL_SIGNAL;
	gpid = pvm_mytid();	
	ComMutex = new AslMutex;
//	ComCondition = new AslCondition;
}


int AslIPCManager::AslGetPort(void)
{
	return 0;
}

char *AslIPCManager::AslGetHost(void)
{
	return "local host";
}


void AslIPCManager::AslScheduler(void)
{
	int res, timeout = 0;

	if(asl_debug)
		aslio<<"ASL IPC MANAGER: In the receiver thread"<<aslend;

//	while(una vuelta a todos los puertos) 
//	{
//		AslCom->recvThread(NULL);
//	}
}

char* AslIPCManager::AslGetAslPath(void)
{
	return aslPath;
}

AslPort *AslIPCManager::AslGetIOPort(int bid)
{
	AslPort *tempNode;

	if (asl_debug)
		aslio<<"ASL IPC MANAGER: Getting Port "<<bid<<" as actual"<<aslend;
	
	PortList.startOnTop();
//		aslio<<"@@@@@ DESPUES DEL PORTLIST.STARTONTOP()"<<aslend;		
	while ((tempNode=PortList.getActual())!=NULL && (tempNode->AslGetBid()!=bid))
		PortList.goNext();

	if (tempNode==NULL)
	{
		aslerr<<"ASL IPC MANAGER ERROR: Can't get Port "<<bid<<aslend;
		return NULL;
	}

	return tempNode;
}

int AslIPCManager::AslGetGpid(void)
{
	return gpid;
}

void AslIPCManager::AslGetEnvironment(void)
{
	aslPath[0]='\0';
	consoleName[0]='\0';
	if (getenv("ASL_HOME")!=NULL)
		strcpy(aslPath, getenv("ASL_HOME"));

	if (getenv("ASL_CONSOLE")!=NULL)
		strcpy(consoleName, getenv("ASL_CONSOLE"));

	if (aslPath[0]=='\0')
		if (getenv("HOME")!=NULL)
			strcpy(aslPath, getenv("HOME"));
	
	if (consoleName[0]=='\0')
		strcpy(consoleName, AslGetHost()); 

	if (asl_debug)
		aslio<<"ASL IPC MANAGER: Binary path ["<<aslPath
		     <<"] Console ["<<consoleName<<"]"<<aslend;
}
// ------------------------------------
int AslIPCManager::AslIncPortID(void)
{
	if (asl_debug)
		aslio<<"ASL IPC MANAGER: Making Port ID with ["<<bidCount<<"]"<<aslend;
	
	int temp = bidCount;
	bidCount++;
	
	return temp;	
}
// ------------------------------------
int AslIPCManager::AslRecv(int bid)
{
	int res;
	int msg = 0;
	AslPort *port = AslGetIOPort(bid);

	if (port==NULL)
	{
		aslerr<<"ASL IPC MANAGER ERROR: Can't find the port with id "<<bid<<aslend;
		cout << "%%%%%%% ERROR EN LA RECEPCION EN ASLRECV()" << endl;
		cout.flush();
		return -1;
	}

	if (!port->AslIsAnInputBuffer())
	{
		aslerr<<"ASL IPC MANAGER ERROR: This is not an input port"<<aslend;
		return -1;
	}
// -----------------------------------------------------------------------------
//	aslio << "->->->->-> ANTES DE PVM_RECV()" << aslend;
	
/*
	ComMutex->AslLock();
	while (msg<=0 && !INTERRUPT)
	{
		ComCondition->AslConditionWait(ComMutex);
//		aslScheduler->AslYieldProcessor();
		msg = pvm_nrecv(-1, bid);
		
//		aslio << "======> ESTOY EN EL WHILE, DESPUES DE PVM_NRECV() MSG:"<<msg<<aslend;
	}
	ComMutex->AslUnlock();
*/

//	aslio << "->->->->-> DESPUES DE PVM_RECV()" << aslend;
	
	ComMutex->AslLock();
        if ((res = pvm_recv(-1 , bid )) >=0)
	{
//		aslio << " %%%%%%% ASL IPC MANAGER: Recibiendo pvm_recv(), con status: "<< res<< aslend;

//		if (asl_debug)
//			aslio << "ASL IPC MANAGER: Receiving Data"<<aslend;
			
                port->AslPVMReceive();
//             	aslio << "->->->->-> DESPUES DE PVM_RECV()" << aslend;
         }
         else
         	aslio << "!!$$$$!!! Error en pvm_recv(..) Status: "<< res << aslend;
         ComMutex->AslUnlock();
// -----------------------------------------------------------------------------

	return 1;
}

// ------------------------------------ 
int AslIPCManager::AslSend(int sourceBid)
{
//	int res=1;
	
	AslPort  *port = AslGetIOPort(sourceBid);
	AslPort *bufSend;	
//	AslPort  *falla = AslGetIOPort(8);

	if (port==NULL)
	{
		aslerr<<"ASL IPC MANAGER ERROR: Can't find the Port with id "<<sourceBid<<aslend;
		return -1;
	}

/*	if (port->AslGetNumberOfElements()==0)
	{
		aslerr<<"ASL IPC MANAGER ERROR: There is nothing in the port to send"<<aslend;
		return -1;
	}
*/	
//	AslBufData *temp = buf->AslGetBufData();
	
	AslPortRelations *connTemp;	
	
	if (port->AslIsAnOutputBuffer())
	{
		port->AslPutConnectionListAtBegining();
		while ((connTemp=port->AslGetConnection())!=NULL)
		{
// -----------------------------------------------------------------------------
			if (connTemp->getProc_id()!= gpid)
	                {
	                	if (asl_debug)
	                        	aslio<<"ASL IPC MANAGER: Sending remote data"<<aslend;

				ComMutex->AslLock();
				pvm_initsend(PvmDataDefault);
	
	                        port->AslPVMSend(1);            // 1- with XDR
                        
	                        pvm_send(connTemp->getProc_id(), connTemp->getPort_id());

	                        ComMutex->AslUnlock();
			}
                	else
	                {
        	        	if (asl_debug)
                	        	aslio<<"ASL IPC MANAGER: Sending local data"<<aslend;

// ----------- VERFICAR ESTE CODIGO CON ESQUEMAS EN EL MISMO PROCESO --------
// -- CON THREADS SI FUNCIONA!!
//			port->AslPVMSend(0);            // 0 - without XDR
//			pvm_send(connTemp->gpid, connTemp->bid);
//		}

// -----------------------------------------------------------------------------			
				bufSend = AslGetIOPort(connTemp->getPort_id());
		
				if (bufSend!=NULL)
					bufSend->AslCopyData(port);
			}
		
			port->AslGoToNextConnection();
		}
	} // del if (AslIsAnOutputBuffer)
// ---------------------- LO MISMO PERO AHORA CON LA LISTA DE RELABELS --------------
	
	port->AslPutRelabelListAtBegining();
	while ((connTemp=port->AslGetRelabel())!=NULL)
	{
		if (connTemp->getProc_id()!= gpid)
                {
                	if (asl_debug)
                        	aslio<<"ASL IPC MANAGER: Sending remote data"<<aslend;

//			cout << "`````` Mandando al puerto:" << connTemp->getPort_id()<<" y gpid:"<< connTemp->getProc_id()<< endl;
			
			ComMutex->AslLock();
			pvm_initsend(PvmDataDefault);
			
                        port->AslPVMSend(1);            // 1- with XDR
                        
                        pvm_send(connTemp->getProc_id(), connTemp->getPort_id());

                        ComMutex->AslUnlock();
		}
                else
                {
                	if (asl_debug)
                        	aslio<<"ASL IPC MANAGER: Sending local data"<<aslend;

			bufSend = AslGetIOPort(connTemp->getPort_id());
		
			if (bufSend!=NULL)
				bufSend->AslCopyData(port);
		}
		
		port->AslGoToNextRelabel();
	}
	return 1;
}
// ------------------------------------
int AslIPCManager::AslConnect(AslPort *src, AslPort *dst)
{
	AslSchema_Pvm *temp;
	
	temp = asl->AslGetSchema(dst->AslGetSchemaAttr()->getname()); // obtaining schema from list
	
	if (src==NULL)
	{
		aslerr<<"ASL IPC MANAGER ERROR: Can't find the Port with id [~] "<<aslend;
		return -1;
	}
	
	if (!src->AslIsAnOutputBuffer() || !dst->AslIsAnInputBuffer())
	{
		aslerr<<"ASL IPC MANAGER ERROR: This is not an output port"<<aslend;
		return -1;
	}

	if(temp->getGpid() == gpid)		// temp->gpid == pvm_mytid()
		dst->setLocalFlag();
	return src->AslAddConnection(dst->AslGetBid(), temp->getGpid());
}

int AslIPCManager::AslDisconnect(AslPort* src, AslPort* dst)
{
	AslSchema_Pvm *temp;
	
	temp = asl->AslGetSchema(dst->AslGetSchemaAttr()->getname());
	
	if (src==NULL)
	{
		aslerr<<"ASL IPC MANAGER ERROR: Can't find the Port with id [~]"<<aslend;
		return -1;
	}

/*	if (!src->AslIsAnOutputBuffer())
	{
		aslerr<<"ASL IPC MANAGER ERROR: This port is not an output port"<<aslend;
		return -1;
	}
*/	
	return src->AslDeleteConnection(dst->AslGetBid(), temp->getGpid());
}

int AslIPCManager::AslRelabel(AslPort *src, AslPort *dst)
{
	AslSchema_Pvm *temp;
	temp = asl->AslGetSchema(dst->AslGetSchemaAttr()->getname());

	if (src==NULL)
	{
		aslerr<<"ASL IPC MANAGER ERROR: Can't find the Port with id [~] "<<aslend;
		return -1;
	}
	
	if (src->AslGetType() != src->AslGetType())
	{
		aslerr<<"ASL IPC MANAGER ERROR: ports are of different type"<<aslend;
		return -1;
	}

	return src->AslAddRelabel(dst->AslGetBid(), temp->getGpid());
}

int AslIPCManager::AslDelabel(AslPort *src, AslPort *dst)
{
	AslSchema_Pvm *temp;
	temp = asl->AslGetSchema(dst->AslGetSchemaAttr()->getname());

	if (src==NULL)
	{
		aslerr<<"ASL IPC MANAGER ERROR: Can't find the Port with id [~]"<<aslend;
		return -1;
	}

/*	if (!src->AslIsAnOutputBuffer())
	{
		aslerr<<"ASL IPC MANAGER ERROR: This port is not an output port"<<aslend;
		return -1;
	}
*/	
	return src->AslDeleteRelabel(dst->AslGetBid(), temp->getGpid());
}

// ------------------------------------
