  
//  ASL RUN TIME SYSTEM LIBRARY
//
//  ASL (Abstract Schema Language) to C++ Compiler
//
//
//
//  FILE NAME   : aslrts_seq.c
//  MODULE TYPE : ASL Runtime System Library
//  STATUS      : In development
//  PURPOSE     : Runtime management 
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyright 1995-1998
//
//
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            0.0 (Apr 20, 1992 - Longguang Wu),
//            Start of development.
//
//            0.5 (May 05, 1993 - Roger Zimmermann),
//            Version 0.5 Alpha.
//
//            0.7 (Dic 02, 1995 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//            1.0 (Aug 12, 1996 - Claudia Calderas & Salvador Marmol), 
//            Version 1.0 Beta.
//
//            1.0 (Oct 23, 1997 - Oscar Peguero), 
//            Version 1.0 Beta 2
//
//            1.1 (Ene 29, 1998 - Oscar Peguero), 
//            Simple version for PC with double buffering (no threads).


#include "nsl_all_kernel_include.h"
#include "xnsl_all_xkernel_include.h"
#include "asl_all_rts_include.h"

// extern void ASLrefresh(void);
// AslCondition schedCond;
AslMutex *schedMutex;

void AslState::setObj(AslSchema *schobj)
{
	sch_obj = schobj; 
//	aslio<< "Setting AslState Object" << sch_obj->getname() << aslend;
}

void AslState::setState(int stat)
{
	state = stat;
	aslio << "Setting AslState state:" << state << aslend;
}

void AslState::setStep(int step)
{
	steps = step; 
//	aslio << "Setting AslState steps:" << steps << aslend;
}

void AslState::setThrFlag(void)
{
	thr_flag = 1;
}

void AslState::unsetThrFlag(void)
{
	thr_flag = 0;
}

void AslState::incStep(void)
{
	steps++; 
	aslio << "Incrementing AslState steps:" << steps << aslend;
}

void AslState::decStep(void)
{
	steps--; 
//	aslio << "Decrementing AslState steps:" << steps << aslend;
}

int AslState::haveThread(void)
{
	return thr_flag;
}

void AslState::wait(void)
{
	stateCond->AslConditionWait(stateSync);
}

void AslState::signal(void)
{
	stateCond->AslConditionSignal();
}

// Methods of class: AslSystem
AslSystem::AslSystem(int argc, char **argv) : nsl_public("asl_system")
{
	gpids = NULL;
	schema_buf = NULL;
	
	strcpy(fileName,"");
//	AslParseArguments(argc,argv); 
	asl_debug=0;	//1;

	if (asl_debug)
		aslio<<"ASL SYSTEM: Kernel initialization"<< aslend;
	
//	cont = 1;
	
	AslCom = new AslIPCManager();
	aslScheduler = new AslThreadManager;
	
//	schedCond = new AslCondition;
	schedMutex = new AslMutex;

//#ifndef	CONSOLE	// necesito que genere un objeto con otro nombre
	if (pvm_parent() != PvmNoParent)	//!=-23) // si es la consola no espera
		AslWaitConsoleTids();
//#endif

// AW98	AslGraphInit(arg);
}

AslSystem::~AslSystem()
{
	delete AslCom;	
	
	if (pvm_parent()!= -23)
	{
		delete[] gpids;
	
		for (int i=0; i<proc_num; i++)
			delete[] schema_buf[i];
		
		delete[] schema_buf;
	}
	
//	delete schedCond;
	delete schedMutex;
}

void AslSystem::AslAddSchema(AslSchema *s)
{	
	MutexSchList.AslLock();
		
	AslSchema_Pvm *temp = new AslSchema_Pvm;
	temp->setObj(s);	//obj = s;
	temp->setGpid(-1);	//gpid = -1; // for knowing if it's already updated
	schemaList.insert(temp);
	
	MutexSchList.AslUnlock();	
}

void AslSystem::AslRemoveSchema(char *nm)
{
	if (asl_debug)
		aslio<<"ASL SYSTEM: Deleting Schema "<<nm<<" from List "<<aslend;
	
	AslSchema_Pvm *temp;
	
	MutexSchList.AslLock();
		
	schemaList.startOnTop();
	while ((temp=schemaList.getActual())!=NULL && (strcmp(temp->getObj()->getname(),nm)!=0))
		schemaList.goNext();
			
	if (temp!=NULL)
	{
		temp=schemaList.removeThis();
		delete temp;
	}
	else
	{
		aslerr<<"ASL SYSTEM ERROR: Can't delete schema from List"<<nm<<aslend;
	}	
	
	MutexSchList.AslUnlock();	
}

/* void AslSystem::AslChangeState(int STATE, int steps)
{	
	AslState *state = new AslState;
	
	state->obj = s;
	state->state = STATE;
	state->steps = steps;
	state->depList.insert(null);
	schemaExecList.insert(state);
}	*/

/* void AslSystem::AslGraphInit(aslArgument arg0)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Graphics initialization" << aslend;
	
//	AslTclGraphInit(arg0);
} */

void AslSystem::AslLoadFile(void)
{	
	if (strcmp(fileName,""))
	{
		if (asl_debug)
			aslio << "ASL SYSTEM: Loading file " <<fileName<< aslend;
	
		// NslLoad(fileName);

	}
}
// -------------------------------------------------------------------------
void AslSystem::AslGeneralScheduler(void)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Executing main schema" << aslend;

//	AslTclGeneralScheduler(this);		
}

void AslSystem::AslDoListStep(void)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Making execution step scheduling" << aslend;
		
	AslState *execst;

	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		AslExecMethod(execst);
		schemaExecList.goNext();
	}
}
// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------
int AslSystem::AslDoListMakeConn(void)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Initiating" << aslend;
		
	AslState *execst;

	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		execst->obj->makeConn();  
		schemaExecList.goNext();
	}
	
	return 1;
}
int AslSystem::AslDoListInitModule(void) 
{
	return 1;
}
int AslSystem::AslDoListEndModule(void)
{
	return 1;
}
int AslSystem::AslDoListInitSys(void)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Initiating" << aslend;
		
	AslState *execst;

	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		breakfg = 1;
		execst->obj->receive();
		execst->obj->initSys();  
		execst->obj->send();
		if (breakfg != 1)
			break;
		schemaExecList.goNext();
	}
	
	return breakfg;
}
int AslSystem::AslDoListEndSys(void)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Initiating" << aslend;
		
	AslState *execst;

	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		breakfg = 1;
		execst->obj->receive();
		execst->obj->endSys();  
		execst->obj->send();
		if (breakfg != 1)
			break;
		schemaExecList.goNext();
	}
	
	return breakfg;
}
	
// run

int AslSystem::AslDoListInitRun(void)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Initiating" << aslend;
		
	AslState *execst;

	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		breakfg = 1;
		execst->obj->receive();
		execst->obj->initRun(); 
		execst->obj->send();
		schemaExecList.goNext();
	}
	
	return breakfg;
}

int AslSystem::AslDoListSimRun(void)
{	
	if (asl_debug)
		aslio << "ASL SYSTEM: Running" << aslend;
		
	AslState *execst;

	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		breakfg = 1;
		execst->obj->receive();
		execst->obj->simRun();  
		execst->obj->send();
		if (breakfg != 1)
			break;
		schemaExecList.goNext();
	}
	
	return breakfg;
}

int AslSystem::AslDoListEndRun(void)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Running" << aslend;
		
	AslState *execst;

	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		breakfg = 1;
		execst->obj->receive();
		execst->obj->endRun(); 
		execst->obj->send();
		schemaExecList.goNext();
	}
	
	return breakfg;
}

// train

int AslSystem::AslDoListInitTrain(void)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Initiating" << aslend;
		
	AslState *execst;

	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		breakfg = 1;
		execst->obj->receive();
		execst->obj->initTrain(); 
		execst->obj->send();
		schemaExecList.goNext();
	}
	
	return breakfg;
}

int AslSystem::AslDoListSimTrain(void)
{	
	if (asl_debug)
		aslio << "ASL SYSTEM: Running" << aslend;
		
	AslState *execst;

	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		breakfg = 1;
		execst->obj->receive();
		execst->obj->simTrain();  
		execst->obj->send();
		if (breakfg != 1)
			break;
		schemaExecList.goNext();
	}
	
	return breakfg;
}

int AslSystem::AslDoListEndTrain(void)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Running" << aslend;
		
	AslState *execst;

	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		breakfg = 1;
		execst->obj->receive();
		execst->obj->endTrain(); 
		execst->obj->send();
		schemaExecList.goNext();
	}
	
	return breakfg;
}

// --------------------------------------------------------------------------------
// --------------------------------------------------------------------------------

void AslSystem::AslDoListConnect(void)
{
	if (asl_debug)
		aslio << "ASL SYSTEM: Connecting" << aslend;
		
	AslState *execst;

	MutexExec.AslLock();
	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
//		if (execst->getState() == READY)
//		{
//			execst->setState(MAKECONN);
			aslScheduler->AslCreateThreadLWP((asl_thread_func_t)FuncThread,(any_t)execst);
//		}
		schemaExecList.goNext();		
	}
	MutexExec.AslUnlock();
}
void AslSystem::AslDoListInit(void)
{
//	AslPort *port = AslCom->AslGetIOPort(8);
//	port->AslPrintConnects();
	
	if (asl_debug)
		aslio << "ASL SYSTEM: Initializing" << aslend;
		
	AslState *execst;
	
	aslio<<"??????????@@@@@@----- ESTOY EN ASLDOLISTINIT"<<aslend;

	MutexExec.AslLock();	
	schemaExecList.startOnTop();
	
//	port->AslPrintConnects();
	
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		execst->getStateSync()->AslLock();
		execst->incStep();
		execst->setState(INITRUN);
//		execst->signal();
		execst->getStateSync()->AslUnlock();

//		aslScheduler->AslYieldProcessor();
			 
		schemaExecList.goNext();
	}
	MutexExec.AslUnlock();	
	
/*	float dumytime=1.23;
	for (int wer=0; wer<9000000; wer++)
		dumytime*=3.214;
*/
	
	MutexExec.AslLock();	
	schemaExecList.startOnTop();
	
	while ((execst = schemaExecList.getActual()) != NULL)
	{
	
		execst->getStateSync2()->AslLock();
		while (execst->getStep() != 1)
			execst->wait2();
		execst->getStateSync2()->AslUnlock();
	
		execst->getStateSync()->AslLock();
		execst->signal();
		execst->getStateSync()->AslUnlock();
		
//		aslScheduler->AslYieldProcessor();
			
		schemaExecList.goNext();
	}
	MutexExec.AslUnlock();	
	
//	float dumytime=1.23;
//	for (int wer=0; wer<9000000; wer++)
//		dumytime*=3.214;
	
//	schedMutex->AslLock();
//	schedCond.AslConditionBroadcast();
//	schedMutex->AslUnlock();
	aslio << "+++++++++ DESPUES DE DoListInit" << aslend;
}

void AslSystem::AslDoListRun(void)
{
	AslState *execst;
	aslio<<"??????????@@@@@@----- ESTOY EN ASLDOLISTRUN"<<aslend;	
//	int pru;
	
	if (asl_debug)
		aslio << "ASL SYSTEM: Running" << aslend;
		
	MutexExec.AslLock();
	schemaExecList.startOnTop();
		
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		execst->getStateSync2()->AslLock();
		while (execst->getStep() != 0)
			execst->wait2();
		execst->getStateSync2()->AslUnlock();

		execst->getStateSync()->AslLock();
		execst->incStep();
		execst->setState(SIMRUN);
//		execst->signal();
		execst->getStateSync()->AslUnlock();
		
//		aslScheduler->AslYieldProcessor();
			
		schemaExecList.goNext();
	}
	MutexExec.AslUnlock();
	
	MutexExec.AslLock();	
	schemaExecList.startOnTop();
	
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		execst->getStateSync()->AslLock();
		execst->signal();
		execst->getStateSync()->AslUnlock();
			
//		aslScheduler->AslYieldProcessor();
		
		schemaExecList.goNext();
	}
	MutexExec.AslUnlock();	
	
	
//	float dumytime=1.23;
//	for (int wer=0; wer<9000000; wer++)
//		dumytime*=3.214;
	
//	schedMutex->AslLock();
//	schedCond.AslConditionBroadcast();	
//	schedMutex->AslUnlock();	
	
	aslio << "+++++++++ DESPUES DE DoListRun" << aslend;		
}

void AslSystem::AslExecMethod(AslState *schema_st)
{	
	if (asl_debug)
		aslio << "ASL SYSTEM: Executing "<<schema_st->getObj()->getname()<<":"<<schema_st->getState()<< aslend;

	switch(schema_st->getState())
	{
		case MAKECONN	: schema_st->getObj()->makeconn();
				  schema_st->setState(schema_st->getState()+1);
				  break;
				  
		case INITRUN	: schema_st->getObj()->initrun();
				  schema_st->setState(schema_st->getState()+1);
				  break;
				  
		case SIMRUN	: int s;
				  if (s = schema_st->getObj()->getSteps()) {
					  schema_st->getObj()->simrun();
					  schema_st->getObj()->setSteps(s-1);
				  }
				  else schema_st->setState(schema_st->getState()+1);
				  break;
				  				  
		case MAKEDISCONN : /* if (schema_st->depList.getActual() == NULL) {
					schema_st->obj->makedisconn();
				   	schema_st->state+=1;
				   }	*/
				   schema_st->setState(schema_st->getState()+1);
				   break;
				   
		case END	:  aslio << "TERMINE ESTADOS" << aslend;
				// AslRemExecSchema(schema_st->obj->getname());
				   break;
	}
}

asl_any_t AslSystem::FuncThread(asl_any_t any_type)
{
	AslState *temp = (AslState *)any_type;
	
	if (temp->getState() == READY)
	{
		temp->setState(MAKECONN);	
		temp->getObj()->makeconn();
	
		temp->setState(READY);
	}
	return NULL;
}

asl_any_t AslSystem::FuncThread2(asl_any_t any_type)
{
	AslState *temp = (AslState *)any_type;
	
//	aslio << "##--##--## BLOQUEADO EN THREAD"<<aslend;

//	asl->MutexExec.AslLock();
	
	temp->getStateSync()->AslLock();
	if (temp->getState() == READY)
	{
//		aslio << "~~~~~~~>>> ENTRE AL IF DE (READY)"<<aslend;

//		asl->MutexExec.AslLock();
		temp->setState(INITRUN);
//		asl->MutexExec.AslUnlock();
		
//		aslio << "~~~~~~~>>> ANTES DE getObj()->initrun()"<<aslend;
		
		temp->getObj()->initrun(); 
		asl->AslSendGraphData("GraphPort");
		
//		aslio << "##--##--## DESPUES DE initrun() EN THREAD"<<aslend;	
	
//		asl->MutexExec.AslLock();
		temp->setState(READY);
//		asl->MutexExec.AslUnlock();
	}
	else
		aslio << "*******>>>>>> NO ENTRO AL IF(READY) EN THREAD" << aslend;

	temp->getStateSync()->AslUnlock();
//	asl->MutexExec.AslUnlock();	

//	aslio << "##--##--## DESPUES DE s-init-etState(READY) EN THREAD"<<aslend;	
	
	return NULL;
}

asl_any_t AslSystem::FuncThread3(asl_any_t any_type)
{
	AslState *temp = (AslState *)any_type;
	
//	aslio << "##--##--## BLOQUEADO EN THREAD"<<aslend;

//	asl->MutexExec.AslLock();

	temp->getStateSync()->AslLock();		
	if (temp->getState() == READY)
	{
//		asl->MutexExec.AslLock();
		temp->setState(SIMRUN);
//		asl->MutexExec.AslUnlock();	
		
		temp->getObj()->simrun();
		asl->AslSendGraphData("GraphPort");	

//		aslio << "##--##--## DESPUES DE simrun() EN THREAD"<<aslend;		
	
//		asl->MutexExec.AslLock();
		temp->setState(READY);
//		asl->MutexExec.AslUnlock();
	}
	else
		aslio << "*******>>>>>>NO ENTRO AL IF(READY) EN THREAD"<<aslend;		

	temp->getStateSync()->AslUnlock();
//	asl->MutexExec.AslUnlock();

//	aslio << "##--##--## DESPUES DE s-sim-etState(READY) EN THREAD"<<aslend;	
	
	return NULL;
}

asl_any_t AslSystem::FuncScheduler(asl_any_t any_type)
{
	AslState *temp = (AslState *)any_type;
//	AslMutex *mutex_tmp;

	/* mutex_tmp =*/ temp->getStateSync()->AslLock();	
	/* mutex_tmp */ 
	do
	{
//		mutex_tmp->AslLock();			
//		aslio << "=====*******==== ESTOY EN CICLO EN THREAD, ANTES DE WAIT" << aslend;
	aslio << "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ ENTRE AL THREAD, STEPS:" << temp->getStep() << aslend;
		while ((temp->getState()!= END) && (temp->getStep() != 1))
		{
			temp->getStateSync2()->AslLock();
			temp->signal2();
			temp->getStateSync2()->AslUnlock();
			
			temp->wait();
		}
			
//		aslScheduler->AslYieldProcessor();
		
//		temp->getStateSync()->AslLock();
		switch (temp->getState())
		{
			case MAKECONN:	temp->getObj()->makeconn();
					break;
					
			case INITRUN:	aslio << "###..###..###..###..#------%%----.%.----%.% ESTOY EN THREAD, INITRUN" << aslend;
					temp->getObj()->initrun();
					break;
					
			case SIMRUN:	aslio << "------------------------------ ESTOY EN THREAD, SIMRUN. STEPS:" << temp->getStep() <<aslend;
					temp->getObj()->simrun();
					break;
					
			case END:	break;
		}
		asl->AslSendGraphData("GraphPort");			
		temp->decStep();
		
		aslio << "=====*******==== ANTES DE WHILE() EN CICLO DE THREAD" << aslend;		
//		mutex_tmp->AslUnlock();
	}	
	while (temp->getState() != END);
	temp->getStateSync()->AslUnlock();	
	
	return NULL;
}
void AslSystem::InitFuncThread(void)
{
	AslState *execst;
	
	MutexExec.AslLock();
	
	schemaExecList.startOnTop();	
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		if (!execst->haveThread())
		{
			execst->setThrFlag();
			CreateFuncSched(execst);
		}
		schemaExecList.goNext();
	}

	MutexExec.AslUnlock();
}

void AslSystem::CreateFuncSched(AslState *execst)
{
	aslScheduler->AslCreateThreadLWP((asl_thread_func_t)FuncScheduler,(any_t)execst);	
}

void AslSystem::FinishFuncThread(void)
{
	AslState *execst;
	
	MutexExec.AslLock();
			
	schemaExecList.startOnTop();
	while ((execst = schemaExecList.getActual()) != NULL)
	{
		execst->getStateSync()->AslLock();
		execst->setState(END);
		execst->signal();
		execst->getStateSync()->AslUnlock();
	
		schemaExecList.goNext();
	}
	
	MutexExec.AslUnlock();
	
//	schedMutex->AslLock();
//	schedCond.AslConditionBroadcast();
//	schedMutex->AslUnlock();	
}
// -------------------------------------------------------------------------

/*void AslSystem::AslParseArguments(int argc, char**argv)
{
	int i;

	arg.argc = argc;
	arg.argv = argv;
	
	asl_debug=0;
	
	for (i=1; i<argc; i++)
	{
		if (!strcmp(argv[i],"-debug"))
			asl_debug=1;
		else if (!strcmp(argv[i],"-graph"))
		{

			i++;
			strcpy(fileName,argv[i]);
		}
		
	}
} */

/* void AslSystem::AslStop(void)
{
	cont = 0;
}
*/
void AslSystem::AslWaitConsoleTids(void)
{
	int item,i,info;
	
	cout << " En AslSystem::AslWaitConsoleTids, mi padre: "<<pvm_parent()<< endl;
		
	pvm_recv(pvm_parent(), INIT);
	
	cout << "Ya recibi del padre !!!" << endl;
	info = pvm_upkint(&item, 1, 1);	// numero de procesos
		
	cout << "Ya desempaque, info="<<info<<" (si INFO=0 => exito)"<< endl;
	
	proc_num = item;
//	proc_num = 2;

	gpids = new int[proc_num];
	
	schema_buf = new char*[proc_num];
	for (i=0 ; i<proc_num; i++)
		schema_buf[i] = new char[256];	
		
//	strcpy(schema_buf[0], "re");
//	strcpy(schema_buf[1], "d1 d2");
	
//	gpids[0] = pvm_mytid();
//	gpids[1] = 786436;
	
	for(i=0; i<proc_num; i++)
	{
		pvm_upkint(&item, 1, 1);
		gpids[i] = item;		// gpid del proceso en donde se ejecutan los sig.
		pvm_upkstr(schema_buf[i]);	// almacena los esquemas de ese gpid
	}
}

AslSchema_Pvm* AslSystem::AslGetSchema(char *sch_name)
{
	AslSchema_Pvm *temp;
	int encontro = 0;
	
	MutexSchList.AslLock();
	
	schemaList.startOnTop();
	while ((temp=schemaList.getActual())!=NULL && !encontro)
	{
		if (!strcmp(temp->getObj()->getname(), sch_name))
		{
			encontro = 1;
//			aslio << "encontro STRCMP(" << temp->getObj()->getname()<< ") en la lista" << aslend;
		}
		else
			schemaList.goNext();
	}
	
//	if (temp==NULL) aslio << "ESQUEMA NO ENCONTRADO EN SCHEMA_LIST" << aslend;
	
	MutexSchList.AslUnlock();

	return temp;
}

int AslSystem::AslLocalExecution(char *sch_name)
{				
	int i,res;
	char *buffer, *item;
	AslSchema_Pvm *temp;
	
	buffer = new char[256];
	item = new char[20];
	res = 0;
	i = 0;
	
//	cout << "@@@@ Estoy en AslSystem::AslLocalExecution("<< sch_name<<")" << endl;
	
	if ((temp = AslGetSchema(sch_name))!= NULL)
	{
		while (i< proc_num && !res)
		{
			strcpy(buffer, schema_buf[i]);	
//			cout << "@@@@ Despues de strcpy(buffer, schema_buf[i]), BUFFER=" << buffer <<endl;
		
			item = strtok(buffer, " ");	// first function's ocurrence
//			cout << "@@@@ Despues de item = strtok(buffer, ' '), ITEM=" << item <<endl;			
		
			if (item != NULL)
			{
				if (!strcmp(item, sch_name))
				{
					temp->setGpid(gpids[i]);	// updating gpid
//					aslio << "!!!!! YA ACTUALIZO EL GPID DE " << temp->getObj()->getname() << aslend;	
					res = 1;
//					break;
				}
			}
			else
				res=-1;

			while ((item = strtok( NULL, " ")) != NULL && !res)
			{
//				aslio << "!!!!! HIZO UN STRTOK EN WHILE, CON ITEM=" << item << aslend;
				if (!strcmp(item, sch_name))
				{	
					temp->setGpid(gpids[i]);
//					aslio << "!!!!! YA ACTUALIZO EL GPID DE " << temp->getObj()->getname() << aslend;
//					break;
					res = 1;
				}
			}
			i++;
		}
		aslio << "temp->gpid: " << temp->getGpid() << " AslCom->AslGetGpid():" << AslCom->AslGetGpid() << aslend;
		if(temp->getGpid() == AslCom->AslGetGpid())
			res=1;
		else
			res=0;
//		aslio << "!!!!! DESPUES DE ASIGNAR, RES=" << res<< aslend;
	}
	else
	{
		aslio << "ASL SYSTEM ERROR: The schema name doesn't match any instance" << aslend;
		res = 0;
	}
	
	if (res == -1)
		return 0;
	else 
		return res;
}

void AslSystem::AslAcceptGraphData(char *port_name) // DEBE ESTAR EN UN THREAD !!
{
	AslSchema_Pvm *temp;
	AslPort *port;
	AslSchema *obj;
	int ocurrence;
	
	MutexSchList.AslLock();
	
	schemaList.startOnTop();
	while ((temp=schemaList.getActual())!=NULL)
	{
// RECIBIRA DE LOS ESQUEMAS REMOTOS PARA HACER LA IMAGEN LOCAL
		obj = temp->getObj();
		ocurrence = obj->AslSearchExternalInOutPortOcurrence(port_name);
		
		for(int i=0; i< ocurrence; i++)
			if ((port = obj->AslGetExternalInOutPort(port_name,i)) != NULL)
			{
//				aslio << " ++||++||++||++ Recibiendo (PORT->RECEIVE)  ++||++||++||++" << aslend;	
				port->receive();
			}
				
		// ASLrefresh();
		schemaList.goNext();
	}

	MutexSchList.AslUnlock();
}

//-=-=-=-=-
void AslSystem::AslSendGraphData(char *port_name) // DEBE ESTAR EN UN THREAD !!
{
	AslSchema_Pvm *temp;
	AslPort *port;
	AslSchema *obj;
	int ocurrence;
	
	MutexSchList.AslLock();
	
	schemaList.startOnTop();
	while ((temp=schemaList.getActual())!=NULL)
	{
		obj = temp->getObj();
		ocurrence = obj->AslSearchExternalInOutPortOcurrence(port_name);
		
		for(int i=0; i< ocurrence; i++)
			if ((port = obj->AslGetExternalInOutPort(port_name,i)) != NULL)
				port->send();
				
		schemaList.goNext();
	}
	
	MutexSchList.AslUnlock();
}

//-=-=-=-=-
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Methods of class: AslSchemaInterface

AslSchemaInterface::AslSchemaInterface (char *str, nsl_model* parent, int fg) : 
	nsl_model(str,parent,fg)
{

	aslExternalOutputPortList = new AslList<AslPort>;
	aslExternalInputPortList  = new AslList<AslPort>;

//	aslExternalInOutPortList  = new AslList<AslPort>;
	aslExternalFuncList = new AslList<AslFunction>;
}

AslSchemaInterface::~AslSchemaInterface (void)
{

	delete aslExternalOutputPortList;
	delete aslExternalInputPortList;

//	delete aslExternalInOutPortList;
}


AslPort* AslSchemaInterface::AslGetExternalInputPort(char *n, int i)
{
	AslPort *temp;
	
	aslExternalInputPortList->startOnTop();
	
	while ((temp=aslExternalInputPortList->getActual())!=NULL && (strcmp(temp->AslGetName(), n) || temp->AslGetIndex()!=i))
		aslExternalInputPortList->goNext();

	return temp;
}

void AslSchemaInterface::AslAddExternalInputPort(AslPort* p)
{
	aslExternalInputPortList->insert(p); 
}

AslPort* AslSchemaInterface::AslGetExternalOutputPort(char* n, int i)
{
	AslPort *temp;
	
	aslExternalOutputPortList->startOnTop();
	
	while ((temp=aslExternalOutputPortList->getActual())!=NULL && (strcmp(temp->AslGetName(), n) || temp->AslGetIndex()!=i))
		aslExternalOutputPortList->goNext();

	return temp;
}

void AslSchemaInterface::AslAddExternalFunc(AslFunction *p)
{ 
	aslExternalFuncList->insert(p);
}

AslFunction* AslSchemaInterface::AslGetExternalFunc(char* n)
{
	AslFunction *temp;
	
	aslExternalFuncList->startOnTop();
	
	while ((temp=aslExternalFuncList->getActual())!=NULL && strcmp(temp->AslGetName(), n))
		aslExternalFuncList->goNext();

	return temp;
}

void AslSchemaInterface::AslAddExternalOutputPort(AslPort*p)
{
	aslExternalOutputPortList->insert(p);
}

/*
AslPort* AslSchemaInterface::AslGetExternalInOutPort(char* n, int i)
{
	AslPort *temp;
	
	aslExternalInOutPortList->startOnTop();
	
	while ((temp=aslExternalInOutPortList->getActual())!=NULL && (strcmp(temp->AslGetName(), n) || temp->AslGetIndex()!=i))
		aslExternalInOutPortList->goNext();

	return temp;
}

void AslSchemaInterface::AslAddExternalInOutPort(AslPort*p)
{
	aslExternalInOutPortList->insert(p);
}
*/
/* asl_any_t AslSchemaInterface::AslExecuteRPC(char *n, asl_any_t arg)
{
	AslFunction *temp;

	temp = AslGetExternalFunc(n);
	return temp->funcPtr(arg);
}	*/
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Methods of class: AslSchema

AslSchema::AslSchema(char *str, nsl_model* parent, int fg) : 
	AslSchemaInterface (str, parent, fg)
{
	strcpy(name, str);

	aslInternalOutputPortList = new AslList<AslPort>;
	aslInternalInputPortList  = new AslList<AslPort>;
	
//	aslInternalInOutPortList = new AslList<AslPort>;

	delegate(this);

//	tid = -1;
	steps = 0;	//stps;	// number of steps for simrun()

// original list
	asl->AslAddSchema(this);
	
	aslio<<"ASL RTS: Creating schema ["<<str<<"]"<<aslend;
}

AslSchema::~AslSchema()
{
	delete aslInternalOutputPortList;
	delete aslInternalInputPortList;

//	delete aslInternalInOutPortList;

	asl->AslRemoveSchema(getname());
	// buscar en la lista de esquemas y borrarse de ahi !!!

/*	if (tid>0)
	{
		aslScheduler->AslGetThreadAnswer(tid, NULL);
		aslScheduler->AslDeleteThread(tid);
	}	*/
}

int AslSchema::display(void)
{		
/*	if (XNSL_SYSTEM != NULL)
	{	
		if (asl_debug)
			aslio<<"ASL SCHEMA: Displaying Graphics"<<aslend;
		XNSL_SYSTEM->display();
	}
	else
		aslerr << "NULL XNSL_SYSTEM" << aslend;
*/	return 1;
}
// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
int AslSchema::nslBreakSim()
{
	if (asl != NULL)
		asl->set_breakfg(-1);
	else
		cmd_out("Null ASL system");

	return 1;
}
int AslSchema::nslBreakEpoch()
{
	if (asl != NULL)
		asl->set_breakfg(-2);
	else
		cmd_out("Null ASL system");

	return 1;
}
// -------------------------------------------------------------------------
// -------------------------------------------------------------------------
void AslSchema::delegate(AslSchema* temp)
{
		// SE DA POR HECHO QUE SE HACE DELEGATE DE TODOS LOS ESQUEMAS !!
	int cont, i;	// Y SIGNIFICA QUE SE ACTUALIZAN LOS GPIDS DE    "
//	va_list ap;	// EN LA LISTA DE ESQUEMAS

        AslState *st = new AslState();

        st->setObj(temp);
        	        
//	st->setState(MAKECONN);	// the first possible schema function
	                			
//	st->setStep(temp->steps);	// necesita de funcion ?????

	if (asl->AslLocalExecution(temp->getname()) )
	{
		asl->getExecList().insert(st);
		aslio << "&&&&& Ya inserte en la lista a "<< st->getObj()->getname() <<aslend;
	}
	else
		delete st;

        asl->InitFuncThread();
}

// -------------------------------------------------------------------------
AslPort *AslSchema::AslGetInternalInputPort(char* n, int i)
{
	AslPort *temp;
	
	aslInternalInputPortList->startOnTop();
	
	while ((temp=aslInternalInputPortList->getActual())!=NULL && (strcmp(temp->AslGetName(), n) || temp->AslGetIndex()!=i))
		aslInternalInputPortList->goNext();

	return temp;
}

void AslSchema::AslAddInternalInputPort(AslPort* p)
{
	aslInternalInputPortList->insert(p); 
}

AslPort *AslSchema::AslGetInternalOutputPort(char* n, int i)
{
	AslPort *temp;
	
	aslInternalOutputPortList->startOnTop();
	
	while ((temp=aslInternalOutputPortList->getActual())!=NULL && (strcmp(temp->AslGetName(), n) || temp->AslGetIndex()!=i))
		aslInternalOutputPortList->goNext();

	return temp;
}

void AslSchema::AslAddInternalOutputPort(AslPort* p)
{
	aslInternalOutputPortList->insert(p);
}

int AslSchema::AslSearchInternalInputPortOcurrence(char *n)
{
	AslPort *temp;
	int cont=0;
	
	aslInternalInputPortList->startOnTop();
	while ((temp=aslInternalInputPortList->getActual())!=NULL)
	{
		if (!strcmp(temp->AslGetName(), n))
			cont ++;
		aslInternalInputPortList->goNext();
	}
		
	return cont;
}

int AslSchema::AslSearchExternalInputPortOcurrence(char *n)
{
	AslPort *temp;
	int cont=0;
	
	aslExternalInputPortList->startOnTop();
	while ((temp=aslExternalInputPortList->getActual())!=NULL)
	{
		if (!strcmp(temp->AslGetName(), n))
			cont ++;
		aslExternalInputPortList->goNext();
	}
		
	return cont;
}

int AslSchema::AslSearchInternalOutputPortOcurrence(const char *n)
{
	AslPort *temp;
	int cont=0;
	
	aslInternalOutputPortList->startOnTop();
	while ((temp=aslInternalOutputPortList->getActual())!=NULL)
	{
		if (!strcmp(temp->AslGetName(), n))
			cont ++;
		aslInternalOutputPortList->goNext();
	}
		
	return cont;
}

int AslSchema::AslSearchExternalOutputPortOcurrence(const char *n)
{
	AslPort *temp;
	int cont=0;
	
	aslExternalOutputPortList->startOnTop();
	while ((temp=aslExternalOutputPortList->getActual())!=NULL)
	{
		if (!strcmp(temp->AslGetName(), n))
			cont ++;
		aslExternalOutputPortList->goNext();
	}
		
	return cont;
}
// -------------------------------------------------------------------------------
// -------------------------------------------------------------------------------
void AslSchema::receive()
{
	AslPort *temp;
	AslList<AslPort>* list;
	
	list = aslExternalInputPortList;
	list->startOnTop();
	while ((temp=(AslPort*)list->getActual())!=NULL) {
		temp->receive();
		list->goNext();
	}
/*
	list = aslInternalInputPortList;
	list->startOnTop();
	while ((temp=(AslPort*)list->getActual())!=NULL){
		temp->receive();
		list->goNext();
	}
	
	list = aslExternalOutputPortList;
	list->startOnTop();
	while ((temp=(AslPort*)list->getActual())!=NULL) {
		temp->receive();
		list->goNext();
	}

	list = aslInternalOutputPortList;
	list->startOnTop();
	while ((temp=(AslPort*)list->getActual())!=NULL){
		temp->receive();
		list->goNext();
	}
*/
}
void AslSchema::send()
{
	AslPort *temp;
	AslList<AslPort>* list;
	
	list = aslExternalOutputPortList;
	list->startOnTop();
	while ((temp=(AslPort*)list->getActual())!=NULL) {
		temp->send();
		list->goNext();
	}

	list = aslInternalOutputPortList;
	list->startOnTop();
	while ((temp=(AslPort*)list->getActual())!=NULL){
		temp->send();
		list->goNext();
	}
	
	list = aslExternalInputPortList;
	list->startOnTop();
	while ((temp=(AslPort*)list->getActual())!=NULL) {
		temp->send();
		list->goNext();
	}

	list = aslInternalInputPortList;
	list->startOnTop();
	while ((temp=(AslPort*)list->getActual())!=NULL){
		temp->send();
		list->goNext();
	}

}

void AslSchema::connect(nsl_layer& src, nsl_layer& dst)
{
	AslPort* psrc = (AslPort*) src.get_port();
	AslPort* pdst = (AslPort*) dst.get_port();
		
	AslSchema_Pvm *temp;
	
// ##### PONER BANDERA DE COMPILACION PARA EL CASO QUE SEA O NO DISTRIBUIDO	
	if((temp = asl->AslGetSchema(psrc->AslGetSchemaAttr()->getname()))!= NULL)
		if (temp->getGpid() == AslCom->AslGetGpid())	// this schema's going to run here
// ##### ENDIF		
			AslCom->AslConnect(psrc, pdst);	
}

void AslSchema::relabel(nsl_layer& src, nsl_layer& dst)
{
	AslPort* psrc = (AslPort*) src.get_port();
	AslPort* pdst = (AslPort*) dst.get_port();
	
// ##### PONER BANDERA DE COMPILACION PARA EL CASO QUE SEA O NO DISTRIBUIDO	
	AslSchema_Pvm *temp;
	
	if((temp = asl->AslGetSchema(psrc->AslGetSchemaAttr()->getname()))!= NULL)
		if (temp->getGpid() == AslCom->AslGetGpid())	// this schema's going to run here
// ##### ENDIF
		
	AslCom->AslRelabel(psrc, pdst);	// verificar para distribuido la variable comm
}

void AslSchema::relabel(AslPort *src, AslPort *dst)
{
// ##### PONER BANDERA DE COMPILACION PARA EL CASO QUE SEA O NO DISTRIBUIDO
	AslSchema_Pvm *temp;
	
	if((temp = asl->AslGetSchema(src->AslGetSchemaAttr()->getname()))!= NULL)
		if (temp->getGpid() == AslCom->AslGetGpid())
// ##### ENDIF

	AslCom->AslRelabel(src, dst);	// verificar para distribuido la variable comm
}

void AslSchema::delabel(AslPort *src, AslPort *dst)
{
// ##### PONER BANDERA DE COMPILACION PARA EL CASO QUE SEA O NO DISTRIBUIDO
	AslSchema_Pvm *temp;
	
	if((temp = asl->AslGetSchema(src->AslGetSchemaAttr()->getname()))!= NULL)
		if (temp->getGpid() == AslCom->AslGetGpid())
// ##### ENDIF

	AslCom->AslDelabel(src, dst);
}

// -------------------------------------------------------------------------------
// -------------------------------------------------------------------------------
void AslSchema::connect(AslPort *src, AslPort *dst)
{
// ##### PONER BANDERA DE COMPILACION PARA EL CASO QUE SEA O NO DISTRIBUIDO	
	AslSchema_Pvm *temp;
	
	if((temp = asl->AslGetSchema(src->AslGetSchemaAttr()->getname()))!= NULL)
		if (temp->getGpid() == AslCom->AslGetGpid())	// this schema's going to run here
// ##### ENDIF
			AslCom->AslConnect(src, dst);	// verificar para distribuido la variable comm
}

void AslSchema::connect(AslPort *src, int remote_bid, int remote_gpid)
{
	src->AslAddConnection(remote_bid, remote_gpid);
}

void AslSchema::disconnect(AslPort *src, AslPort *dst)
{
// ##### PONER BANDERA DE COMPILACION PARA EL CASO QUE SEA O NO DISTRIBUIDO
	AslSchema_Pvm *temp;
	
	if((temp = asl->AslGetSchema(src->AslGetSchemaAttr()->getname()))!= NULL)
		if (temp->getGpid() == AslCom->AslGetGpid())
// ##### ENDIF
			AslCom->AslDisconnect(src, dst);
}

void AslSchema::disconnect(AslPort *src, int remote_bid, int remote_gpid)
{
	src->AslDeleteConnection(remote_bid, remote_gpid);
}

void AslSchema::nslGraphics(AslPort *sch_port)
{
	AslSchema_Pvm *temp;
	AslPort *temp_port;
	
//	GraphPortU = new AslInOutPort(this, AslCom, "GraphPortU", EXTERNAL, dis_var);
//	GraphPortM = new AslInOutPort(this, AslCom, "GraphPortM", EXTERNAL, dis_var);

//#ifndef CONSOLE
	if (pvm_parent()!= -23)
	{
//		aslio << "=======> ESTOY EN NSLGRAPHICS (CONSOLA), CONECTO LOS PG"<<aslend;
		
		temp_port = sch_port;
		temp_port->AslSetType(ASL_DATA_OUTPUT);

//		if((temp = asl->AslGetSchema(getname()))!= NULL)
//			if(temp->getGpid() == AslCom->AslGetGpid())
				connect(temp_port, temp_port->AslGetBid(), pvm_parent());
	}
//#endif
}

AslPort *AslSchema::AslGetInternalInOutPort(char* n, int i)
{
	AslPort *temp;
	
	aslInternalInOutPortList->startOnTop();
	
	while ((temp=aslInternalInOutPortList->getActual())!=NULL && (strcmp(temp->AslGetName(), n) || temp->AslGetIndex()!=i))
		aslInternalInOutPortList->goNext();

	return temp;
}

void AslSchema::AslAddInternalInOutPort(AslPort* p)
{
	aslInternalInOutPortList->insert(p); 
}

int AslSchema::AslSearchInternalInOutPortOcurrence(char *n)
{
	AslPort *temp;
	int cont=0;
	
	aslInternalInOutPortList->startOnTop();
	while ((temp=aslInternalInOutPortList->getActual())!=NULL)
	{
		if (!strcmp(temp->AslGetName(), n))
			cont ++;
		aslInternalInOutPortList->goNext();
	}
		
	return cont;
}

int AslSchema::AslSearchExternalInOutPortOcurrence(char *n)
{
	AslPort *temp;
	int cont=0;
	
	aslExternalInOutPortList->startOnTop();
	while ((temp=aslExternalInOutPortList->getActual())!=NULL)
	{
		if (!strcmp(temp->AslGetName(), n))
			cont ++;
		aslExternalInOutPortList->goNext();
	}
		
	return cont;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Methods of class: AslFunction
AslFunction::AslFunction(AslSchema *s, char *n, int arg, int ans /*, asl_thread_func_t f */)
{
	schema = s;
	strcpy(name, n);
	argType = arg;
	answerType = ans;
//	funcPtr = f;
	
	aslio << "antes de agregarlo \n";
	schema->AslAddExternalFunc(this);
	aslio << "despues de agregarlo \n";

	if (asl_debug)
		aslio << "ASL RTS: Adding function "<<n<<aslend;
} 

char* AslFunction::AslGetName(void)
{
	return name;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
AslPort::AslPort(const char *str, nsl_public *s, int com)
{
	schema = (AslSchema*) s;	// se puede definir con el metodo AslsetSchema 
	m_parent = s;	
	
	strcpy(name, str);
	comType = com;	// ASL_DATA_OUTPUT/INPUT...
	
	scope = EXTERNAL;
	
	if (comType==ASL_DATA_INPUT)	
		((AslSchema*) s)->AslAddExternalInputPort(this);
	else if (comType==ASL_DATA_OUTPUT)
		((AslSchema*) s)->AslAddExternalOutputPort(this);
	else
		aslio << "ASL RTS: Not a valid port type: " << comType << aslend;
	
	aslComm = AslCom;		//antes c en lugar de comm

	index = AslSearchPortOcurrence(str);
	if (asl_debug)
		aslio << "ASL RTS: Creating Port "<<str<<"["<<index<<"]"<<aslend;
			
	AslCom->getPortList().insert(this);
		
	bid = aslComm->AslIncPortID();
	
	numberOfMessages = 0;
	numberOfConnections = 0;
	numberOfRelabels = 0;	
	
	LOCAL = 0;
} 

AslPort::~AslPort(void) 
{
/*	aslComm->AslDelBuf(bid);	*/
}

int AslPort::AslSearchPortOcurrence(const char *n) 
{
	if (scope==INTERNAL)
		return schema->AslSearchInternalInputPortOcurrence(n);
	else
		return schema->AslSearchExternalInputPortOcurrence(n);
}

int AslPort::AslGetIndex(void) 
{ 
	return index; 
}

int AslPort::AslGetType(void) 
{ 
	return comType; 
}

void AslPort::AslSetType(int type_id) 
{ 
	comType = type_id;
}

int AslPort::AslGetScope(void) 
{ 
	return scope; 
}

int AslPort::AslGetBid(void) 
{ 
	return bid; 
}

char* AslPort::AslGetName(void) 
{ 
	return name; 
}

void AslPort::AslsetSchema(AslSchema *s) 
{ 
	schema = s;
}
// -----------------
int AslPort::AslIsAnInputBuffer(void)
{
	return (comType==ASL_DATA_INPUT);
//	aslio << "!!!!!!! comType=" << comType << aslend;
//	return (comType==ASL_DATA_INPUT) || (comType==ASL_CONTROL_INPUT);
//	return 1;
}

int AslPort::AslIsAnOutputBuffer(void)
{
	return (comType==ASL_DATA_OUTPUT);
//	return (comType==ASL_DATA_OUTPUT) || (comType==ASL_CONTROL_OUTPUT) 
//	    || (comType==ASL_NEW_CONTAINER) || (comType==ASL_RELABEL_BUFFER);
//	return 1;
}

int AslPort::AslGetNumberOfElements(void)
{
	int res;
	
	res = numberOfMessages;

	return res;
}

int AslPort::AslAddConnection(int bidc, int rgpid)
{
	AslPortRelations *temp;
	AslPortRelations *node;
//	char sch_name[20];	// name of schema for connect with
//	int rschema;

	node = new AslPortRelations;
	
	node->setPort_id(bidc);
	node->setProc_id(rgpid);
	
	connections.startOnTop();
	while ((temp=connections.getActual())!=NULL && (temp->getPort_id()!=bidc || temp->getProc_id()!=rgpid))
		connections.goNext();
	
	if (temp==NULL)
	{
		numberOfConnections++;	
		connections.insert(node);
		if (asl_debug)
			aslio<<"ASL PORT: I have connected local buffer ["<< bid 
		    	<<"] with ["<< node->getPort_id()<< "] and gpid ["<< node->getProc_id()<< "]" << aslend;
	}
	else
		if (asl_debug)
			aslio<<"ASL PORT: Connection was already established"<<aslend;	
	
//	AslPrintConnects();
	
	return 1;
}

int AslPort::AslDeleteConnection(int bidc, int rgpid)
{
	int res=1;
	
	AslPortRelations *temp;
	
	connections.startOnTop();
	while ((temp=connections.getActual())!=NULL && temp->getPort_id()!=bidc && temp->getProc_id()!=rgpid)
		connections.goNext();
	
	if (temp==NULL)
	{
		aslerr<<"ASL PORT ERROR: I can't disconnect buffer ["<<bid
		      <<"] of buffer ["<<bidc<<"] "<<aslend;
		res=0;
	}
	else
	{
		numberOfConnections--;
		temp = connections.removeThis();
		delete temp;
	}

	return res;	
}

AslPortRelations* AslPort::AslGetConnection(void)
{
	return connections.getActual();
}

void AslPort::AslPutConnectionListAtBegining(void)
{
	connections.startOnTop();
}

void AslPort::AslGoToNextConnection(void)
{
	connections.goNext();
}
// -------------------------------------------------------------------------------
// -------------------------------------------------------------------------------
int AslPort::AslAddRelabel(int bidc)
{
	AslPortRelations *temp, *node = new AslPortRelations;
	
	node->bid = bidc;
	
	relabels.startOnTop();
	while ((temp=relabels.getActual())!=NULL && temp->bid!=bidc)
		relabels.goNext();
	
	if (temp==NULL)
	{
		numberOfRelabels++;	
		relabels.insert(node);
	}
	else
		if (asl_debug)
			aslio<<"ASL IPC MANAGER: Relabel was already established"<<aslend;		    
	
	if (asl_debug)
		aslio<<"ASL IPC MANAGER: I have relabeled local buffer with bid ["<< bid 
		    <<"] with ["<< bidc << "] "<<aslend;
		    
	return 1;
}

int AslPort::AslDeleteRelabel(int bidc)
{
	int res=1;
	
	AslPortRelations *temp;
	
	relabels.startOnTop();
	while ((temp=relabels.getActual())!=NULL && temp->bid!=bidc)
		relabels.goNext();
	
	if (temp==NULL)
	{
		aslerr<<"ASL PORT ERROR: I can't delabel buffer ["<<bid
		      <<"] of buffer ["<<bidc<<"] "<<aslend;
		res=0;
	}
	else
	{
		numberOfRelabels--;
		temp = relabels.removeThis();
		delete temp;
	}

	return res;	
}

AslPortRelations* AslPort::AslGetRelabel(void)
{
	return relabels.getActual();
}

void AslPort::AslPutRelabelListAtBegining(void)
{
	relabels.startOnTop();
}

void AslPort::AslGoToNextRelabel(void)
{
	relabels.goNext();
}

void AslPort::send()
{
}

void AslPort::receive()
{
}
// -------------------------------------------------------------------------------
// -------------------------------------------------------------------------------
/*void AslPort::AslPrintConnects(void)
{
	AslPortRelations *temp;
	
	connections.startOnTop();
	while ((temp=connections.getActual())!=NULL)
	{
		aslio << "--------Puerto:"<< bid << " -->" << temp->getPort_id() << "," << temp->getProc_id()
		<< aslend;
		connections.goNext();
	}
	if (temp==NULL)
	{
		aslio << "--------Puerto:" << bid << " NO TIENE MAS CONEXIONES" << aslend;
	}
} */
