/* House.asl 	House.nslm according to the NSL 3.0 specifications	21.4.98 

	Updates:
		- NslModule& to NslModule*
		- *this to this
		- this in all the constructors
		- member initialization
*/


extern nslModule NslModule;



nslClass RetinaMethods ()
{
	float xyview_to_retina(float x, float y, float w, float yf, float l, float rg) 
	{
		float b,q,a0;
		a0 = atan(w/(yf+l));
		b  = atan((float)(x - rg*w)/(y+l));
		q  = M_2_PI*(b = rg*a0);
		return (q + 1.0);
	}
};



nslModule Dev (NslString st, NslModule* p, int sizeX, int sizeY) extends NslModule (st,p) 
{
	private: NslFloat2 s("s",sizeX,sizeY);
	private: NslFloat2 t("t",sizeX,sizeY);
	private: NslFloat2 m_pot("m_pot",sizeX,sizeY);
	private: NslFloat2 m_fire("m_fire",sizeX,sizeY);
	private: NslFloat1 u_pot("u_pot",sizeY);
	private: NslFloat1 u_fire("u_fire",sizeY);

	public: NslDinFloat2 sp("sp",s);
	public: NslDinFloat2 tp("tp",t);
	public: NslDoutFloat2 mp("mp",m_fire);		
	private: NslFloat0 ksm("*");
	private: NslFloat0 kmu("*");
	private: NslFloat0 kum("*");
	private: NslFloat0 ktm("*");
	private: NslFloat1 wm("*",3);
	private: NslFloat0 m_tm("*");
	private: NslFloat0 u_tm("*");
	private: NslFloat0 m_h("*");
	private: NslFloat0 u_h("*");
	private: NslFloat0 x1("*"); 
	private: NslFloat0 x2("*");

	
	public: void initRun ()
	{
		m_pot  = 0;
		m_fire = 0;
		u_pot  = 0;
		u_fire = 0;
	}


	public: void simRun()
	{
		m_pot  = nslDiff(m_pot, m_tm, -m_pot + ksm*s - kum*u_fire + ktm*t + (wm opConv m_fire) + m_h);
		m_fire = nslSigmoid(m_pot,x1,x2);
		u_pot  = nslDiff(u_pot, u_tm, -u_pot + kmu*m_fire + u_h);
		u_fire = nslRamp(u_pot);
	}

};



nslModule Retina (NslString st, NslModule* p, int sizeX, int sizeY, float rfg) extends NslModule (st,p)
{
	private: NslFloat2 in("in",sizeX,sizeY);
	private: NslFloat1 r("r",(2*sizeY)-1);

	public: NslDinFloat2 inp("inp",in);
	public: NslDoutFloat2 rp("rp",r);

	private: NslFloat0 w("*");
	private: NslFloat0 yf("*");
	private: NslFloat0 l("*");
	private: NslFloat rg=rfg;

	private: RetinaMethods rm();

	public: void initRun()
	{
		int iq0, iq1;
		float val, q0, q1;
		float xc, yc, xc0, xc1;

		int imax  = in.get_imax();
		int jmax  = in.get_jmax();
		
		int yz    = imax/2;
		int xz    = jmax/2;

		float dx  = 1.0;
		float dy  = 1.0;

		int rimax = r.get_imax();
		int midq  = (rimax - rimax%2)/2;

		float k   = 0.5;
		float wx2 = 0.5;
		
		for (int j=0;j<jmax;j++)
		  for(int i=0;i<imax;i++)
			{
			  	val = in.elem(i,j);
				if (val >= k)
				  {
					xc  = (j-xz)*dx;
					yc  = (i-yz)*dy;
					xc0 = xc - wx2;
					xc1 = xc + wx2;
					q0  = rm.xyview_to_retina(xc0,yc,w,yf,l,rg);
					iq0 = rint(midq*q0);
					q1  = rm.xyview_to_retina(xc1,yc,w,yf,l,rg);
					iq1 = rint(midq*q1);
					vectorFill(r,iq0,iq1);
				  }
			}

	}

	private: void vectorFill(NslFloat1& q,int i0, int i1)
	{
		for(int i=i0;i<=i1;i++) q.elem(i) = 1.0;
	}


};



nslModule Accommodation (NslString st, NslModule* p, int sizeX, int sizeY, int sizeR) extends NslModule (st,p)
{
	private: NslFloat2 in("in",sizeX,sizeY);
	private: NslFloat2 a("a",sizeX,sizeY);

	public: NslDinFloat2 inp("inp",in);
	public: NslDoutFloat2 ap("rp",a);

	private: NslFloat0 w("*");
	private: NslFloat0 yf("*");
	private: NslFloat0 l("*");
	private: NslFloat0 dmax("*");
	private: NslFloat0 sigma("*");
	private: NslFloat0 rsize("*")=sizeR;	

	private: RetinaMethods rm();

	public: void initRun()
	{
		float da,q1,qr;
		float d,r,s,val;
		int id,ip;
		float xc,yc,xc0,xc1,x;

		int imax     = a.get_imax();
		int jmax     = a.get_jmax();
		
		int yz       = imax/2;
		int xz       = jmax/2;
	
		float dx     = 1.0;
		float dy     = 1.0;

		int rimax    = rsize;
		int midq     = (rimax - rimax%2)/2;
		int midy     = (imax - imax%2)/2;
		int midx     = (jmax - jmax%2)/2;

		float dqdi   = 1.0/(jmax-1);
		float dddi   = 1.0/(imax-1);
		float dpdi   = dqdi*midq/midx;
		float dipdiq = dqdi/dpdi;
		float k	     = 0.5;
		float xs     = 0.5;
		float wx2    = 0.5;
		
		for(int j=0;j<jmax;j++)
		  for(int i=0;i<imax;i++)
			{
				val = in.elem(i,j);
				if (val >= k)
				  {
					xc  = (j-xz)*dx;
					yc  = (i-yz)*dy;
					xc0 = xc - wx2;
					xc1 = xc + wx2;
					for(x=xc0;x<=xc1;x+=xs)
					  {
						q1 = rm.xyview_to_retina(x,yc,w,yf,l,-1.0);
						qr = rm.xyview_to_retina(x,yc,w,yf,l,1.0);
						da = qr-q1;
						ip = rint((float)(midq*(qr-1.0)*dipdiq));
						for(id= -midy;id<=midy;id++)
						  {
							d = id*dddi;
							r = pow((d-da)/(sigma*dmax),2.0);
							s = exp(-r/2.0);
							a.elem(id+midy,ip+midx) = s;
						  }
					  }
				  }
		    	}

	}

};



nslModule Disparity (NslString st, NslModule* p, int sizeX,  int sizeY) extends NslModule (st,p)
{
	private: NslFloat2 rl("rl",(2*sizeY)-1);
	private: NslFloat2 rr("rr",(2*sizeY)-1);
	private: NslFloat2 d("d",sizeX,sizeY);

	public: NslDinFloat2 rlp("rlp",rl);
	public: NslDinFloat2 rrp("rrp",rr);
	public: NslDoutFloat2 dp("dp",d);

	public: void initRun ()
	{
		int iq,id,iql,ip;
		float val;
		
		int imax     = d.get_imax();
		int jmax     = d.get_jmax();
		int midy     = (imax - imax%2)/2;
		int midx     = (jmax - jmax%2)/2;
		int rimax    = rr.get_imax();
		int midq     = (rimax - rimax%2)/2;
		
		float dqdi   = 1.0/(jmax-1);
		float dddi   = 1.0/(imax-1);
		float dpdi   = dqdi*midq/midx;
		float dipdiq = dqdi/dpdi;
		for(iq= -midq;iq<=midq;iq++)
		  {
			ip = rint(iq*dipdiq);
			for(id= -midy;id<=midy;id++)
			  {
				iql = iq - id;
				if (iql >= -midq && iql <= midq)
				  {
					val = rl.elem(iql+midq)*rr.elem(iq+midq);
					if (d.elem(id+midy,ip+midx) == 0.0) d.elem(id+midy,ip+midx) = val;
				  }
			  }		
			
		  }
		
	}
	
};



nslModule RetinaMap (NslString st, NslModule* p, int sizeX, int sizeY) extends NslModule(st,p)
{
	private: Retina rr("rr",this,sizeX,sizeY,+1.0);
	private: Retina rl("rr",this,sizeX,sizeY,-1.0);
	private: Disparity d("d",this,sizeX,sizeY);
	private: Accommodation a("a",this,sizeX,sizeY);

	public: NslDinFloat2 inp("inp",sizeX,sizeY);
	public: NslDoutFloat2 ap("ap",sizeX,sizeY);
	public: NslDoutFloat2 dp("dp",sizeX,sizeY);

	public: void makeConn ()
	{
		connect(rl.rp,d.rlp);
		connect(rr.rp,d.rrp);
		relabel(inp,rl.inp);
		relabel(inp,rr.inp);
		relabel(inp,a.inp);
		relabel(ap,a.ap);
		relabel(dp,d.dp);
	}

};



nslModule Stereo (NslString st, NslModule* p, int sizeX, int sizeY) extends NslModule (st,p)
{
	private: RetinaMap rm("*",this,sizeX,sizeY);
	private: Dev m("*",this,sizeX,sizeY), s("*",this,sizeX,sizeY);
	
	public: NslDinFloat2 inp("inp",sizeX,sizeY);
	public: NslDinFloat2 up("up",sizeX,sizeY);

	public: void makeConn()
	{
		connect(rm.ap,m.ap);
		connect(rm.dp,s.dp);
		connect(m.up,s.tp);
		connect(s.up,m.tp);
		relabel(inp,rm.inp);
		relabel(up,s.up);	
	}
};



nslModule Visin (NslString st, NslModule* p, int sizeX, int sizeY) extends NslModule (st,p)
{
	private: NslFloat2 in("in",sizeX,sizeY);
	private: NslDoutFloat2 inp("inp",in);
};



nslModule NslMainModule (NslString st, NslModule* p) extends NslModule (st,p)
{

	private: Visin vis("*",this,81,81);
	private: Stereo ste("*",this,81,81);

	public: void makeConn()
	{
		connect(vis.inp,st.inp);
	}
};

