
//  ASL B-TREE LIBRARY
//
//  ASL (Abstract Schema Language) B-Tree Library
//
//
//
//  FILE NAME   : aslbtree.h
//  MODULE TYPE : ASL B-Tree Library
//  STATUS      : Ready for use
//  PURPOSE     : General B-Tree Library
//
//
//  Instituto Tecnologico Autonomo de Mexico
//  (c) Copyrigth 1995
//
//
//
//  HISTORY : Version (Date-Name) Modification, description.
//
//            1.0 (Dec 20, 1995 - Claudia Calderas & Salvador Marmol),
//            Start of development for distributed execution.

#pragma interface

#ifndef __ASL_BTREE
#define __ASL_BTREE 1

#include <stdio.h>

class AslPtr
{
};

template <class T>
class AslBTreeNode : public AslPtr
{
public:
	T   *object;
	int  eqFact;
	AslBTreeNode<T> *ptrLeft;
	AslBTreeNode<T> *ptrRight;
public:
	AslBTreeNode(void);
	~AslBTreeNode(void);
};


template <class T>
class AslBTree
{
	AslBTreeNode<T>  *ptrRoot;
	AslBTreeNode<T>  *ptrActual;
	T *ptrThis;

	int   count;
	int   found;

	void  bSearch(AslBTreeNode<T>*, T*);
	void  bInsert(AslBTreeNode<T>**, T*, int&);
	void  bRemove(AslBTreeNode<T>**, T*, int&);

	void  erase(AslBTreeNode<T>**, AslBTreeNode<T>**, int&);
	void  restructLeft(AslBTreeNode<T>**, int&);
	void  restructRight(AslBTreeNode<T>**, int&);

public:
	int   insert(T*);
	T*    search(T*);
	int   remove(T*);

	void startOnTop(void);
	void goLeft(void);
	void goRight(void);
	void setActualPtr(AslPtr*);
	AslPtr* getActualPtr(void);

	T*    getActual(void);

	int   numberOfElements(void);
	int   isEmpty(void);

	AslBTree(void);
	~AslBTree(void);
};

template <class T> AslBTreeNode<T>::AslBTreeNode(void)
{
	ptrLeft=NULL;
	ptrRight=NULL;
	eqFact = 0;
}

template <class T> AslBTreeNode<T>::~AslBTreeNode(void)
{
}

template <class T> AslBTree<T>::AslBTree(void)
{
	ptrRoot=NULL;
	count=0;
}

template <class T> AslBTree<T>::~AslBTree(void)
{
	delete ptrRoot;
}

template <class T> void AslBTree<T>::startOnTop(void)
{
	ptrActual = ptrRoot;
}

template <class T> void AslBTree<T>::setActualPtr(AslPtr* ptr)
{
	ptrActual = (AslBTreeNode<T>*)ptr;
}

template <class T> AslPtr* AslBTree<T>::getActualPtr(void)
{
	return (AslPtr*)ptrActual;
}

template <class T> void AslBTree<T>::goLeft(void)
{
	if (ptrActual!=NULL)
		ptrActual = ptrActual->ptrLeft;
}

template <class T> void AslBTree<T>::goRight(void)
{
	if (ptrActual!=NULL)
		ptrActual = ptrActual->ptrRight;
}

template <class T> T* AslBTree<T>::getActual(void)
{
	if (ptrActual!=NULL)
		return ptrActual->object;
	else
		return NULL;
}

template <class T> int AslBTree<T>::numberOfElements(void)
{
	return count;
}

template <class T> int AslBTree<T>::isEmpty(void)
{
	if (count==0)
		return 1;
	else
		return 0;
}

template <class T> void AslBTree<T>::erase(AslBTreeNode<T> **node1, AslBTreeNode<T> **node2, int &hasStretch)
{
	if ((*node1)->ptrRight != NULL)
	{
		erase(&((*node1)->ptrRight), node2, hasStretch);
		if (hasStretch)
			restructRight(node1, hasStretch);
	}
	else
	{
		(*node2)->object = (*node1)->object;
		*node1 = (*node1)->ptrLeft;
		hasStretch = 1;
	}
}

template <class T> void AslBTree<T>::restructLeft(AslBTreeNode<T> **node, int &hasStretch)
{
	AslBTreeNode<T> *node1, *node2;

	switch( (*node)->eqFact)
	{
		case -1: (*node)->eqFact = 0;
			break;
		case 0: (*node)->eqFact = 1;
			hasStretch = 0;
			break;
		case 1: node1 = (*node)->ptrRight;
			 if (node1->eqFact>=0)
			 {
				(*node)->ptrRight = node1->ptrLeft;
				node1->ptrLeft = *node;
				switch (node1->eqFact)
				{
					case 0: (*node)->eqFact = 1;
						node1->eqFact = -1;
						hasStretch = 0;
					case 1: (*node)->eqFact = 0;
						node1->eqFact = 0;
				}
				*node = node1;
			 }
			 else
			 {
				node2 = node1->ptrLeft;
				(*node)->ptrRight = node2->ptrLeft;
				node2->ptrLeft = *node;
				node1->ptrLeft = node2->ptrRight;
				node2->ptrRight = node1;
				if (node2->eqFact == 1)
					(*node)->eqFact = -1;
				else
					(*node)->eqFact = 0;
				if (node2->eqFact == -1)
					node1->eqFact = 1;
				else
					node1->eqFact = 0;
				*node = node2;
				node2->eqFact = 0;
			 }
			 break;
	}
}

template <class T> void AslBTree<T>::restructRight(AslBTreeNode<T> **node, int &hasStretch)
{
	AslBTreeNode<T> *node1, *node2;

	switch( (*node)->eqFact)
	{
		case 1: (*node)->eqFact = 0;
			break;
		case 0: (*node)->eqFact = -1;
			hasStretch = 0;
			break;
		case -1: node1 = (*node)->ptrLeft;
			 if (node1->eqFact<=0)
			 {
				(*node)->ptrLeft = node1->ptrRight;
				node1->ptrRight = *node;
				switch (node1->eqFact)
				{
					case 0: (*node)->eqFact = -1;
						node1->eqFact = 1;
						hasStretch = 0;
					case -1: (*node)->eqFact = 0;
						node1->eqFact = 0;
				}
				*node = node1;
			 }
			 else
			 {
				node2 = node1->ptrRight;
				(*node)->ptrLeft = node2->ptrRight;
				node2->ptrRight = *node;
				node1->ptrRight = node2->ptrLeft;
				node2->ptrLeft = node1;
				if (node2->eqFact == -1)
					(*node)->eqFact = 1;
				else
					(*node)->eqFact = 0;
				if (node2->eqFact == 1)
					node1->eqFact = -1;
				else
					node1->eqFact = 0;
				*node = node2;
				node2->eqFact = 0;
			 }
			 break;
	}
}


template <class T> void AslBTree<T>::bRemove(AslBTreeNode<T> **node, T* data, int &hasStretch)
{
	if (*node != NULL)
	{
		if (*data < *((*node)->object))
		{
			bRemove( &((*node)->ptrLeft), data, hasStretch);
			if (hasStretch)
				restructLeft(node, hasStretch);
		}
		else
		{
			if (*data > *((*node)->object))
			{
				bRemove( &((*node)->ptrRight), data, hasStretch);
				if (hasStretch)
					restructRight(node, hasStretch);
			}
			else
			{
				AslBTreeNode<T> *temp;
				temp = *node;
				if (temp->ptrRight == NULL)
					*node=temp->ptrLeft;
				else
				{
					if (temp->ptrLeft == NULL)
						*node=temp->ptrRight;
					else
					{
						erase(&(temp->ptrLeft), &temp, hasStretch);
						if (hasStretch)
							restructLeft(node, hasStretch);
					}
				}
				// delete temp; //The element must be deleted
				count--;
				found = 1;
			}
		}
	}
	else
		found = 0;
}

template <class T> void AslBTree<T>::bInsert(AslBTreeNode<T> **node, T* data, int &hasGrew)
{
	AslBTreeNode<T> *node1, *node2;

	if (*node != NULL)
	{
		if (*data < *((*node)->object) )
		{
			bInsert( &((*node)->ptrLeft), data, hasGrew);
			if (hasGrew)
			{
				switch( (*node)->eqFact)
				{
					case 1: (*node)->eqFact = 0;
						hasGrew = 0;
						break;
					case 0: (*node)->eqFact = -1;
						break;
					case -1: node1 = (*node)->ptrLeft;
						 if (node1->eqFact<=0)
						 {
							(*node)->ptrLeft = node1->ptrRight;
							node1->ptrRight = *node;
							(*node)->eqFact = 0;
							*node = node1;
						 }
						 else
						 {
							node2 = node1->ptrRight;
							(*node)->ptrLeft = node2->ptrRight;
							node2->ptrRight = *node;
							node1->ptrRight = node2->ptrLeft;
							node2->ptrLeft = node1;
							if (node2->eqFact == -1)
								(*node)->eqFact = 1;
							else
								(*node)->eqFact = 0;
							if (node2->eqFact == 1)
								node1->eqFact = -1;
							else
								node1->eqFact = 0;
							*node = node2;
						 }
						 (*node)->eqFact = 0;
						 hasGrew = 0;
						 break;
				}
			}
		}
		else
		{
			if (*data > *((*node)->object))
			{
				bInsert(&((*node)->ptrRight), data, hasGrew);
				if (hasGrew)
				{
					switch((*node)->eqFact)
					{
						case -1: (*node)->eqFact = 0;
							hasGrew = 0;
							break;
						case 0: (*node)->eqFact = 1;
							break;
						case 1: node1 = (*node)->ptrRight;
							 if (node1->eqFact>=0)
							 {
								(*node)->ptrRight = node1->ptrLeft;
								node1->ptrLeft = *node;
								(*node)->eqFact = 0;
								*node = node1;
							 }
							 else
							 {
								node2 = node1->ptrLeft;
								(*node)->ptrRight = node2->ptrLeft;
								node2->ptrLeft = *node;
								node1->ptrLeft = node2->ptrRight;
								node2->ptrRight = node1;
								if (node2->eqFact == 1)
									(*node)->eqFact = -1;
								else
									(*node)->eqFact = 0;
								if (node2->eqFact == -1)
									node1->eqFact = 1;
								else
									node1->eqFact = 0;
								*node = node2;
							 }
							 (*node)->eqFact = 0;
							 hasGrew = 0;
							 break;
					}
				}
			}
			else
				found = 1;
		}
	}
	else
	{
		AslBTreeNode<T> *temp;
		temp = new AslBTreeNode<T>;
		temp->object = data;
		hasGrew=1;
		*node = temp;
		count++;
		found = 0;
	}
}

template <class T> int AslBTree<T>::insert(T *data)
{
	int hasGrew=0;
	bInsert(&ptrRoot, data, hasGrew);
	return found;
}

template <class T> int AslBTree<T>::remove(T *data)
{
	int hasStretch=0;
	bRemove(&ptrRoot, data, hasStretch);
	return found;
}

template <class T> void AslBTree<T>::bSearch(AslBTreeNode<T> *node, T *data)
{
	if (node!=NULL)
		if (*data < *(node->object) )
			bSearch(node->ptrLeft, data);
		else
			if (*data > *(node->object) )
				bSearch(node->ptrRight, data);
			else
				ptrThis = node->object;
	else
		ptrThis = NULL;
}

template <class T> T* AslBTree<T>::search(T *data)
{
	bSearch(ptrRoot, data);
	return ptrThis;
}

#endif
