%{

//  ASL COMPILER
//
//  ASL (Abstract Schema Language) to c++ compiler
//
//
//
//  FILE NAME   : aslcyacc.y
//  MODULE TYPE : Yacc input specification
//  STATUS      : In development
//  PURPOSE     : Yacc source to generate ASL parser
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyrigth 1995-1997 
//
//
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            0.00  (Apr 20, 1992 - Longguang Wu), 
//            Start of development.
//
//            0.50  (May 05, 1993 - Roger Zimmermann),
//            Version 0.5 alpha.     
//
//            1.00  (Dec 17, 1995 - Claudia Calderas & Salvador Marmol), 
//            Compiler reconstruction, sintax extended.
//
//	      1.05  (Oct 09, 1997 - Oscar Peguero),
//	      Sintax according to the NSLC 3.0 specifications.
//
//	      1.10  (Feb 16, 1998 - Sebastian Gutierrez),
//	      Sintax extended according to the NSLC 3.0 specifications.
// 

#include "aslc.h"

void yyerror(char* s)
{
	/* compiler.sintaxError(s); */
}

%}

%start schema

%union
{
   char  name[STR_LEN_LINE];     // String constants and identifiers.
}

/* Constants */

%token  <name>  CONST_CHAR 
%token  <name>  CONST_INT
%token  <name>  CONST_FLOAT
%token  <name>  CONST_STRING


/* Simple types */

%token  <name>  INT 
%token  <name>  CHAR
%token  <name>  DOUBLE 
%token  <name>  FLOAT 
%token  <name>  VOID
%token  <name>  SHORT 
%token  <name>  LONG 
%token  <name>  SIGNED 
%token  <name>  UNSIGNED


/* Nsl data types */

%token  <name>  NSLDINDOUBLE
%token  <name>  NSLDINDOUBLE1
%token  <name>  NSLDINDOUBLE2
%token  <name>  NSLDINDOUBLE3
%token  <name>  NSLDINDOUBLE4
%token  <name>  NSLDINFLOAT
%token  <name>  NSLDINFLOAT1
%token  <name>  NSLDINFLOAT2
%token  <name>  NSLDINFLOAT3
%token  <name>  NSLDINFLOAT4
%token  <name>  NSLDININT
%token  <name>  NSLDININT1
%token  <name>  NSLDININT2
%token  <name>  NSLDININT3
%token  <name>  NSLDININT4
%token  <name>  NSLDOUTDOUBLE
%token  <name>  NSLDOUTDOUBLE1
%token  <name>  NSLDOUTDOUBLE2
%token  <name>  NSLDOUTDOUBLE3
%token  <name>  NSLDOUTDOUBLE4
%token  <name>  NSLDOUTFLOAT
%token  <name>  NSLDOUTFLOAT1
%token  <name>  NSLDOUTFLOAT2
%token  <name>  NSLDOUTFLOAT3
%token  <name>  NSLDOUTFLOAT4
%token  <name>  NSLDOUTINT
%token  <name>  NSLDOUTINT1
%token  <name>  NSLDOUTINT2
%token  <name>  NSLDOUTINT3
%token  <name>  NSLDOUTINT4
%token  <name>  NSLDOUBLE
%token  <name>  NSLDOUBLE0
%token  <name>  NSLDOUBLE1
%token  <name>  NSLDOUBLE2
%token  <name>  NSLDOUBLE3
%token  <name>  NSLDOUBLE4
%token  <name>  NSLFLOAT
%token  <name>  NSLFLOAT0
%token  <name>  NSLFLOAT1
%token  <name>  NSLFLOAT2
%token  <name>  NSLFLOAT3
%token  <name>  NSLFLOAT4
%token  <name>  NSLINT
%token  <name>  NSLINT0
%token  <name>  NSLINT1
%token  <name>  NSLINT2
%token  <name>  NSLINT3
%token  <name>  NSLINT4
%token  <name>  NSLSTRING


/* Type and variable names */

%token  <name>  SCHEMA_NAME
%token  <name>  STRUCT_NAME
%token  <name>  FUNCTION_NAME
%token  <name>  PORT_NAME
%token  <name>  IDENTIFIER_DECLARED
%token  <name>  IDENTIFIER


/* C++ tokens */

%token  <name>  AUTO
%token  <name>  BREAK
%token  <name>  CASE
%token  <name>  CONST
%token  <name>  CONTINUE
%token  <name>  DEFAULT
%token  <name>  DELETE
%token  <name>  DO
%token  <name>  ELSE
%token  <name>  ENUM
%token  <name>  EXTERN
%token  <name>  FOR
%token  <name>  IF
%token  <name>  NEW
%token  <name>  REGISTER
%token  <name>  RETURN
%token  <name>  SIZEOF
%token  <name>  STATIC
%token  <name>  SWITCH
%token  <name>  VOLATILE
%token  <name>  WHILE


/* C++ operators */

%token  <name>  MULT_EQUAL          /*  "*="  */
%token  <name>  DIV_EQUAL           /*  "/="  */
%token  <name>  MOD_EQUAL           /*  "%="  */
%token  <name>  PLUS_EQUAL          /*  "+="  */
%token  <name>  MINUS_EQUAL         /*  "-="  */
%token  <name>  SHIFT_RIGHT_EQUAL   /*  ">>=" */
%token  <name>  SHIFT_LEFT_EQUAL    /*  "<<=" */
%token  <name>  AND_EQUAL           /*  "&="  */
%token  <name>  XOR_EQUAL           /*  "^="  */
%token  <name>  OR_EQUAL            /*  "|="  */
%token  <name>  OR_LOGIC            /*  "||"  */
%token  <name>  AND_LOGIC           /*  "&&"  */
%token  <name>  EQUAL               /*  "=="  */
%token  <name>  NOT_EQUAL           /*  "!="  */
%token  <name>  SHIFT_LEFT          /*  "<<"  */
%token  <name>  SHIFT_RIGHT         /*  ">>"  */
%token  <name>  LOWER_EQUAL         /*  "<="  */
%token  <name>  GREATER_EQUAL       /*  ">="  */
%token  <name>  PLUS_PLUS           /*  "++"  */
%token  <name>  MINUS_MINUS         /*  "--"  */
%token  <name>  INDIRECTION_1       /*  "."   */
%token  <name>  INDIRECTION_2       /*  ".*"  */
%token  <name>  INDIRECTION_3       /*  "->*" */


/* ASL tokens */

%token  <name>  DELEGATE
%token  <name>  EXTERNAL 
%token  <name>  INHERIT
%token  <name>  INTERNAL
%token  <name>  INPUT 
%token  <name>  OUTPUT
%token  <name>  SCHEMA
%token  <name>  SELF
%token  <name>  SIN 
%token  <name>  SOUT
%token  <name>  STOP
%token  <name>  THIS


/* ASL operators */

%token  <name>  OP_CONNECT_LR	/*  ">=>"  */ 
%token  <name>  OP_CONNECT_RL	/*  "<=<"  */
%token  <name>  OP_DISCONNECT 	/*  ">=<"  */
%token  <name>  OP_RELABEL 	/*  "==="  */
%token  <name>  OP_DELABEL	/*  "=|="  */


/* Types */

%type <name> declSpecifierList
%type <name> declSpecifier
%type <name> simpleTypeName
%type <name> nslTypeName
%type <name> classStorageSpecifier
%type <name> typeSpecifier
%type <name> accessOption
%type <name> pointerOperator
%%

/*

ASL syntax according to:

	"ASL: A Hierarchical Computational Model for Distributed Heterogeneous Systems" 
	 by Alfredo Weitzenfeld.

ASL syntax extended according to:

	"Hierarchical Model for Distributed Schemas"
	 by Claudia Calderas & Salvador Marmol



	Shift/Reduce conflicts:

		- Classic nested if then else problem.
		- Pointers' vector and simple pointers.
		- Casting an expression and declaring something;
*/

/* Schema declaration */

schema			: schemaSpecifier	{ 
						  compiler.parser.setLexState(0);        
						  compiler.parser.incNumOfSchemas();
						  flagAssign = 2;
						}

			| SCHEMA 
			  IDENTIFIER 		{ compiler.parser.insertSchema($2,TRUE); }
			  ';' 			{ compiler.parser.setLexState(0);flagAssign = 2; }

			| SCHEMA 
			  SCHEMA_NAME
			  ';' 			{ compiler.parser.setLexState(0);flagAssign = 2; }
						

			| schema 
			  schemaSpecifier 	{
						  compiler.parser.setLexState(0);
						  compiler.parser.incNumOfSchemas();
						  flagAssign = 2;
						}

			| schema 
			  SCHEMA 
			  IDENTIFIER 		{ compiler.parser.insertSchema($3,TRUE); }
			  ';' 			{ compiler.parser.setLexState(0);flagAssign = 2; }

			| schema 
			  SCHEMA
			  SCHEMA_NAME
			  ';' 			{ compiler.parser.setLexState(0);flagAssign = 2; }
						  			  
			| accessSpecifier
			  schema
			;

schemaSpecifier		: schemaHeader 		
			  '{' 			{
						  compiler.parser.makeConstructorDecl(compiler.parser.getActualSchema());
					  	  compiler.parser.makeDestructorDecl(compiler.parser.getActualSchema());
					  	  strcpy(myAssign,"");
						}
			  memberList 
			  '}' 			
			  ';'			{
						  compiler.parser.makeConstructor(compiler.parser.getActualSchema());
					  	  compiler.parser.makeDestructor(compiler.parser.getActualSchema());
						}
			;

schemaHeader		: SCHEMA		
			  IDENTIFIER 		{
						  flagSchema=TRUE;
						  compiler.parser.insertSchema($2,FALSE); 
						  compiler.parser.setActualSchema($2);
						}
			  construct_args	{ flagSchema=FALSE;}	
			  baseSpecifier		{ flagSchema=FALSE;}

			| SCHEMA 	
			  SCHEMA_NAME 		{
						  flagSchema=TRUE;
						  AslSchemaSymbol *res;
						  res = compiler.parser.searchSchema($2);
						  if (!res->isPrototype())
							{
							  sprintf(compiler.buf,"Schema [%s] already defined", $2);
							  compiler.semanticError(compiler.buf);
							}
						  else
							{
							  res->setPrototype(FALSE);
							} 
						}

			  construct_args	{ flagSchema=FALSE;}
			  baseSpecifier		{ flagSchema=FALSE;}
			   
			| error			{ compiler.sintaxError("Schema Header sintax error"); }

			;

construct_args		: /* optional */	{ flagSchema=FALSE; }
			| '(' 
			  argumentList 
			  ')'			{ flagSchema=FALSE; }
			;


baseSpecifier		: /* optional */ 		{ compiler.parser.setActualExtends(""); flagSchema=FALSE;}
			| INHERIT 			{ flagSchema=FALSE; flagObject=FALSE;}
			  baseList 
			  extendsArgs  			{ flagSchema=FALSE;}
			;

baseList		: SCHEMA_NAME			{ compiler.parser.setActualExtends($1);
							  flagSchema=TRUE;
							}

			| IDENTIFIER 			{ 
						  	  sprintf(compiler.buf,"Identifier [%s] is not a schema", $1);
					   	  	  compiler.semanticError(compiler.buf);
							}

			| baseList ',' IDENTIFIER 	{ 
							  sprintf(compiler.buf,"Identifier [%s] is not a schema", $3);
							  compiler.semanticError(compiler.buf);
				     			}
			;


extendsArgs		: /* optional */
			| '('				{flagExtends = TRUE;}
			  argumentList
			  ')'				{flagExtends = FALSE;}
			;

memberList		: /* optional */
			| memberDeclaration memberList
			| accessSpecifier memberList
			| error			{ compiler.sintaxError("MyBad schema member sintax"); }
			;

accessSpecifier		: EXTERNAL		{ compiler.parser.setAccess(EXT); }
			| INTERNAL		{ compiler.parser.setAccess(INT); }
			;


/* Member declarations */


memberDeclaration	: declSpecifierList
			  declMemberList	
			  ';'			{ if (flagObject) flagObject = FALSE; 
			  			  if (flagAssign) flagAssign = FALSE; 
			  			}

			| declSpecifierList 	
			  declMemberList	
			  '{'			{ compiler.parser.insertFunction(compiler.parser.getActualIdentifier()); }
			  statementList		
			  '}'
			;

declSpecifierList	: declSpecifier				{ strcpy($$, $1); compiler.parser.setActualType($$); }
			| declSpecifierList declSpecifier 	{ strcat($$, $2); compiler.parser.setActualType($$); }
			;

declSpecifier		: classStorageSpecifier		{ strcpy($$, ""); }
			| typeSpecifier 	 	{ strcpy($$, $1); }
			;

classStorageSpecifier	: AUTO			{ strcpy($$, ""); }
			| REGISTER		{ strcpy($$, ""); }
			| STATIC		{ strcpy($$, ""); }
			| EXTERN		{ strcpy($$, ""); }
			;

accessOption		: /*optional */			{ strcpy($$, ""); compiler.parser.setActualAccessOption($$); }
			| pointerOperator		{ compiler.parser.setActualAccessOption($$);}
			;

typeSpecifier		: simpleTypeName accessOption	{ strcpy($$, $1); }
			| nslTypeName accessOption 	{ strcpy($$, $1);flagObject= TRUE;}
			| CONST				{ strcpy($$, ""); }
			;

simpleTypeName		: CHAR 		{ strcpy($$, "char"); 	  }
			| DOUBLE 	{ strcpy($$, "double");   }
			| FLOAT		{ strcpy($$, "float"); 	  }
			| INPUT 	{ strcpy($$, "input"); 	  }
			| INT 		{ strcpy($$, "int"); 	  }
			| LONG 		{ strcpy($$, "long"); 	  }
			| NSLSTRING	{ strcpy($$, "NslString");}
			| OUTPUT 	{ strcpy($$, "output");   }
			| SIGNED 	{ strcpy($$, "signed");   }
			| SHORT 	{ strcpy($$, "short"); 	  }
			| UNSIGNED 	{ strcpy($$, "unsigned"); }
			| VOID		{ strcpy($$, "void");     }
			;

nslTypeName		: NSLDINDOUBLE	 	{ strcpy($$, "NslDinDouble"); 	}
			| NSLDINDOUBLE1	 	{ strcpy($$, "NslDinDouble1"); 	}
			| NSLDINDOUBLE2	 	{ strcpy($$, "NslDinDouble2"); 	}
			| NSLDINDOUBLE3	 	{ strcpy($$, "NslDinDouble3"); 	}	
			| NSLDINDOUBLE4	 	{ strcpy($$, "NslDinDouble4"); 	}
			| NSLDINFLOAT	 	{ strcpy($$, "NslDinFloat"); 	}
			| NSLDINFLOAT1	 	{ strcpy($$, "NslDinFloat1");	}
			| NSLDINFLOAT2	 	{ strcpy($$, "NslDinFloat2"); 	}	
			| NSLDINFLOAT3	 	{ strcpy($$, "NslDinFloat3"); 	}
			| NSLDINFLOAT4	 	{ strcpy($$, "NslDinFloat4"); 	}
			| NSLDININT	 	{ strcpy($$, "NslDinInt"); 	}
			| NSLDININT1	 	{ strcpy($$, "NslDinInt1"); 	}
			| NSLDININT2	  	{ strcpy($$, "NslDinInt2"); 	}
			| NSLDININT3	 	{ strcpy($$, "NslDinInt3"); 	}
			| NSLDININT4	 	{ strcpy($$, "NslDinInt4"); 	}
			| NSLDOUTDOUBLE	 	{ strcpy($$, "NslDoutDouble"); 	}
			| NSLDOUTDOUBLE1 	{ strcpy($$, "NslDoutDouble1"); }
			| NSLDOUTDOUBLE2 	{ strcpy($$, "NslDoutDouble2"); }
			| NSLDOUTDOUBLE3 	{ strcpy($$, "NslDoutDouble3"); }	
			| NSLDOUTDOUBLE4 	{ strcpy($$, "NslDoutDouble4"); }
			| NSLDOUTFLOAT	 	{ strcpy($$, "NslDoutFloat"); 	}
			| NSLDOUTFLOAT1 	{ strcpy($$, "NslDoutFloat1"); 	}
			| NSLDOUTFLOAT2	 	{ strcpy($$, "NslDoutFloat2"); 	}	
			| NSLDOUTFLOAT3	 	{ strcpy($$, "NslDoutFloat3"); 	}
			| NSLDOUTFLOAT4	 	{ strcpy($$, "NslDoutFloat4"); 	}
			| NSLDOUTINT	 	{ strcpy($$, "NslDoutInt"); 	}
			| NSLDOUTINT1	 	{ strcpy($$, "NslDoutInt1"); 	}
			| NSLDOUTINT2	 	{ strcpy($$, "NslDoutInt2"); 	}
			| NSLDOUTINT3	 	{ strcpy($$, "NslDoutInt3"); 	}
			| NSLDOUTINT4	 	{ strcpy($$, "NslDoutInt4"); 	}
			| NSLDOUBLE	 	{ strcpy($$, "NslDouble"); 	}
			| NSLDOUBLE0	 	{ strcpy($$, "NslDouble0"); 	}
			| NSLDOUBLE1	 	{ strcpy($$, "NslDouble1"); 	}
			| NSLDOUBLE2	 	{ strcpy($$, "NslDouble2"); 	}
			| NSLDOUBLE3	 	{ strcpy($$, "NslDouble3"); 	}
			| NSLDOUBLE4	 	{ strcpy($$, "NslDouble4"); 	}
			| NSLFLOAT 		{ strcpy($$, "NslFloat"); 	}
			| NSLFLOAT0 		{ strcpy($$, "NslFloat0"); 	}
			| NSLFLOAT1 		{ strcpy($$, "NslFloat1"); 	}
			| NSLFLOAT2 	 	{ strcpy($$, "NslFloat2"); 	}
			| NSLFLOAT3	 	{ strcpy($$, "NslFloat3"); 	}
			| NSLFLOAT4	 	{ strcpy($$, "NslFloat4"); 	}
			| NSLINT	 	{ strcpy($$, "NslInt"); 	}
			| NSLINT0	 	{ strcpy($$, "NslInt0"); 	}
			| NSLINT1	 	{ strcpy($$, "NslInt1"); 	}
			| NSLINT2	 	{ strcpy($$, "NslInt2"); 	}
			| NSLINT3	 	{ strcpy($$, "NslInt3"); 	}
			| NSLINT4	 	{ strcpy($$, "NslInt4"); 	}
			| SCHEMA_NAME		{ strcpy($$, $1);}
			;


declMemberList		: /* optional */			
			| memberDecl		
			| declMemberList 
			  ',' 			{ if (flagObject) compiler.putInOutFile(","); 
						  flagAssign=TRUE;
						  flagSchema=TRUE;
						}
			  memberDecl		
			;


memberDecl		: declarator  	{ if (flagObject) compiler.parser.insertObject(compiler.parser.getActualType(), compiler.parser.getActualAccessOption(), compiler.parser.getActualIdentifier());
					}
				 	
			  '(' 			{ if (flagObject) strcpy(myArgument,"");}
			 initializerList	{ if (flagObject) compiler.parser.insertObjectArguments(myArgument);}
			  ')' 			
			 myInit 
				
			| declarator pureSpecifier				

			| IDENTIFIER ':' constantExpression			

			| ':' constantExpression

			| declarator '=' 	{ 
				  	  	  if (flagAssign)
						    {
							strcat(myAssign,compiler.parser.getActualIdentifier());
							strcat(myAssign," = ");
						    }
						}
			  constNeg constantExpression	
			;

pureSpecifier		: /* optional */ 
			| '=' CONST_INT { if (!strcmp($2, "0")) compiler.sintaxError("Pure specifier need to be '= 0'"); }
			;

myInit			: /* optional */
			| '=' 				{ 
				  	  	  	  if (flagAssign) 
				  	  	  	    {
				  	  	  	    	strcat(myAssign,compiler.parser.getActualIdentifier());
				  	  	  	    	strcat(myAssign," = ");
				  	  	  	    }
							}
			  constNeg constantExpression	
			;

declarator		: IDENTIFIER 			{  	
								compiler.parser.insertIdentifier($1);
							}

			| IDENTIFIER_DECLARED		{ 
							  if (!flagExtends)
							    {		
								sprintf(compiler.buf,"MyIdentifier [%s] already defined", $1);
								compiler.semanticError(compiler.buf);
							    }
							}

			| pointerOperator declarator

			| declarator '[' constantExpression ']'

			| declarator '[' ']'

			| '(' declarator ')'		
			;

pointerOperator		: '*' cvQualifierList		{strcpy($$,"*");}
			| '&' cvQualifierList		{strcpy($$,"&");}
			;

cvQualifierList		: /* optional */
			| cvQualifier cvQualifierList
			;

cvQualifier		: CONST
			| VOLATILE
			;

argument		: /* optional */
			| argsDef		{
						  compiler.parser.insertArg(compiler.parser.getActualType(), compiler.parser.getActualAccessOption(), compiler.parser.getActualIdentifier(), flagExtends);
						}
			| objectInitList
			| objectMethodsInvList 
			;

argsDef			: declSpecifierList
			| declSpecifierList declarator 
			| declSpecifierList declarator '=' constantExpression
			| declarator		
			;

argumentList		: argument
			| argumentList ',' argument
			; 

objectInitList		: objectInit
			| objectInitList objectInit
			;

objectInit		: declarator '=' constNeg constantExpression ';'
			;

constNeg		: /* optional*/		
			| '-'			{ if (flagAssign) strcat(myAssign,"-"); }
			;

objectMethodsInvList	: objectMethodsInv
			| objectMethodsInvList objectMethodsInv
			;

objectMethodsInv	: IDENTIFIER INDIRECTION_1 memberDecl ';'
			;

typeName		: typeSpecifierList
			; 

typeSpecifierList	: /* optional */
			| typeSpecifier typeSpecifierList
			;

/* Declarations */

declaration		: declSpecifierList declaratorList ';'
			/*| declaratorList ';'*/
			;

declaratorList		: initDeclarator
			| declaratorList ',' initDeclarator
			;

initDeclarator		: declarator initializer
			;

initializer		: /* optional */
			| '=' assignExpression
			| '=' '{' initializerList ',' '}'
			| '=' '{' initializerList '}'
			| '(' expressionList ')'
			;

initializerList		:  /* optional */
			
			| assignExpression
			
			| initializerList 
			  ',' 				{if (flagObject) strcat(myArgument,",");}
			  assignExpression

			| '{' initializerList ',' '}'

			| '{' initializerList '}'
			;

/* Expressions */ 

expression		: portExpression
			| expressionIter
			;

portExpression		: STOP '(' schemaList ')'
			| DELEGATE '(' schemaList ')'
			| PORT_NAME portOperator PORT_NAME
			| PORT_NAME portOperator '(' portList ')'
			;

portOperator		: OP_CONNECT_LR      /*  ">=>" */ 
			| OP_CONNECT_RL	     /*  "<=<" */ 
			| OP_DISCONNECT      /*  ">=<" */
			| OP_RELABEL         /*  "===" */
			| OP_DELABEL	     /*  "=|=" */
			| SHIFT_RIGHT        /*  ">>"  */
			| SHIFT_LEFT         /*  "<<"  */
			;

portList		: PORT_NAME
			| portList ',' PORT_NAME
			;

schemaList		: SCHEMA_NAME
			| schemaList ',' SCHEMA_NAME
			;

expressionIter		: assignExpression			
			| expression ',' assignExpression	
			;

assignExpression	: conditionalExpression					
			| unaryExpression assignOperator assignExpression	
			;

assignOperator		: '='
			| MULT_EQUAL         /*  "*="  */
			| DIV_EQUAL          /*  "/="  */
			| MOD_EQUAL          /*  "%="  */
			| PLUS_EQUAL         /*  "+="  */
			| MINUS_EQUAL        /*  "-="  */
			| SHIFT_RIGHT_EQUAL  /*  ">>=" */
			| SHIFT_LEFT_EQUAL   /*  "<<=" */
			| AND_EQUAL          /*  "&="  */
			| XOR_EQUAL          /*  "^="  */
			| OR_EQUAL           /*  "|="  */
			;

conditionalExpression	: orLogicalExpression						
			| orLogicalExpression '?' expression ':' conditionalExpression
			;

orLogicalExpression	: andLogicalExpression
			| orLogicalExpression OR_LOGIC andLogicalExpression
			;

andLogicalExpression	: inOrExpression
			| andLogicalExpression AND_LOGIC inOrExpression
			;

inOrExpression		: xorExpression
			| inOrExpression '|' xorExpression
			;

xorExpression		: andExpression
			| xorExpression '^' andExpression
			;

andExpression		: equalExpression
			| andExpression '&' equalExpression
			;

equalExpression		: relationalExpression
			| equalExpression EQUAL     relationalExpression
			| equalExpression NOT_EQUAL relationalExpression
			;

relationalExpression	: moveExpression
			| relationalExpression '<'  moveExpression
			| relationalExpression '>'  moveExpression
			| relationalExpression LOWER_EQUAL   moveExpression
			| relationalExpression GREATER_EQUAL moveExpression
			;

moveExpression		: aditiveExpression
			| moveExpression SHIFT_LEFT  aditiveExpression
			| moveExpression SHIFT_RIGHT aditiveExpression
			;

aditiveExpression	: prodExpression
			| aditiveExpression 
			  '+' 			{ if (flagObject) { strcat(myArgument,"+");	}
			  			  if (flagAssign) { strcat(myAssign,"+");	}
			  			}
  			  prodExpression
			| aditiveExpression
			  '-' 			{ if (flagObject) { strcat(myArgument,"-");	}
			  			  if (flagAssign) { strcat(myAssign,"-");	}
			  			}
			  prodExpression
			;

prodExpression		: apmExpression
			| prodExpression 
			  '*' 			{ if (flagObject) { strcat(myArgument,"*");	}
			  			  if (flagAssign) { strcat(myAssign,"*");	}
			  			}
			  apmExpression	
			| prodExpression
			  '/' 			{ if (flagObject) { strcat(myArgument,"/");	}
			  			  if (flagAssign) { strcat(myAssign,"/");	}
			  			}
			  apmExpression
			| prodExpression
			  '%' 			{ if (flagObject) { strcat(myArgument,"%");	}
			  			  if (flagAssign) { strcat(myAssign,"%");	}
			  			}
			  apmExpression
			;


apmExpression		: mutationExpression
			| apmExpression INDIRECTION_2 mutationExpression
			| apmExpression INDIRECTION_3 mutationExpression
			;

mutationExpression	: unaryExpression						
			| '(' typeName ')' mutationExpression		
			;

unaryExpression		: postfixExpression			
			| PLUS_PLUS   unaryExpression
			| MINUS_MINUS unaryExpression
			| unaryOperator mutationExpression
			| SIZEOF unaryExpression
			| SIZEOF '(' typeName ')'
			| newExpression
			| deleteExpression
			;

unaryOperator		: '*'			{ if (flagObject) { strcat(myArgument,"*");	}
						  if (flagAssign) { strcat(myAssign,"*");	}
						}
			| '&'			{ if (flagObject) { strcat(myArgument,"&");	}
						  if (flagAssign) { strcat(myAssign,"&");	}
						}
			| '+'			{ if (flagObject) { strcat(myArgument,"+");	}
						  if (flagAssign) { strcat(myAssign,"+");	}
						}
			| '-'			{ if (flagObject) { strcat(myArgument,"-");	}
						  if (flagAssign) { strcat(myAssign,"-");	}
						}
			| '!'			{ if (flagObject) { strcat(myArgument,"!");	}
						  if (flagAssign) { strcat(myAssign,"!");	}
						}
			| '~'			{ if (flagObject) { strcat(myArgument,"~");	}
						  if (flagAssign) { strcat(myAssign,"~");	}
						}
			;

newExpression		: NEW typeName '(' initializerList ')'
			| NEW typeName
			;

deleteExpression	: DELETE mutationExpression
			| DELETE '[' ']' mutationExpression
			;

postfixExpression	: primaryExpression					
			| postfixExpression '[' expression ']'			
			| postfixExpression '(' expressionList ')'		
			| postfixExpression INDIRECTION_1 IDENTIFIER			
			| postfixExpression PLUS_PLUS				
			| postfixExpression MINUS_MINUS
			| simpleTypeName accessOption primaryExpression		/* AT_SCHEMA en medio */
			| nslTypeName accessOption primaryExpression		/* AT_SCHEMA en medio */
			;

expressionList		: /* optional */
			| assignExpression
			| expressionList ',' assignExpression
			;

primaryExpression	: literal	
			
			| SELF			{ 
						  if (flagObject) 
						    {
							strcat(myArgument,"\"");
							strcat(myArgument,compiler.parser.getActualIdentifier());
							strcat(myArgument,"\"");
						    }
						  else
						    {
							compiler.putInOutFile("\"");
							compiler.putInOutFile(compiler.parser.getActualIdentifier());
							compiler.putInOutFile("\"");
						    }
						}
			
			| THIS			{ if (flagObject) strcat(myArgument,myStack);}
		
			| '(' 			{ if (flagObject) strcat(myArgument,"(");
						  if (flagAssign) strcat(myAssign,"(");
						}
			  assignExpression
			  ')'			{ if (flagObject) strcat(myArgument,")");
			  			  if (flagAssign) strcat(myAssign,")");
			  			}
		
			| IDENTIFIER_DECLARED	{ if (flagObject) strcat(myArgument,$1);
						  if (flagAssign) strcat(myAssign,$1);
						} /* | AT_SCHEMA principio */	

			| IDENTIFIER		{ if (flagObject) strcat(myArgument,$1);
						  if (flagAssign) strcat(myAssign,$1);
						} /* | AT_SCHEMA principio */
			;

literal			: CONST_INT		{ if (flagObject) strcat(myArgument,myStack);}
			| CONST_CHAR		{ if (flagObject) strcat(myArgument,myStack);}
			| CONST_FLOAT		{ if (flagObject) strcat(myArgument,myStack);}
			| CONST_STRING		{ if (flagObject) strcat(myArgument,myStack);}
			;

constantExpression	: conditionalExpression
			;

/* Statements */

statement		: labelStatement
			| initForStatement
			| complexStatement    
			| selectStatement
			| iterationStatement
			| jumpStatement
			| error			/*{ compiler.sintaxError("Bad statement sintax"); }*/
			;

labelStatement		: CASE constantExpression ':'
			  statement
			| DEFAULT ':'
			  statement
			;	

expressionStatement	: optExpression ';'
			;

complexStatement	: '{' 			{ compiler.parser.incScope(); }
			  statementList 
			  '}' 			{ compiler.parser.decScope(); }
			;

statementList		: /* optional */
			| statementList statement
			;

selectStatement		: IF '(' expression ')' statement
			| IF '(' expression ')' statement ELSE statement
			| SWITCH '(' expression ')' statement
			; 

iterationStatement	: WHILE '(' expression ')' statement
			| DO statement WHILE '(' expression ')' ';'
			| FOR '(' initForStatement optExpression ';' optExpression ')' statement
			;

optExpression		: /* optional */
			| expression
			;

initForStatement	: expressionStatement
			| declarationStatement 
			;

jumpStatement		: BREAK ';'
			| CONTINUE ';'
			| RETURN expression ';'
			| RETURN ';'
			;

declarationStatement	: declaration 
			;

%%
