/* SCCS  @(#)nsl_sys_main.C	1.1---95/08/13--21:10:41 */
/**************************************************************/
/*                                                            */
/*                       nsl_sys_main.C                       */
/*                                                            */
/**************************************************************/

# include "nsl_all_kernel_include.h"

# ifdef NSL_INPUT_FG
	# include "nsl_all_input_include.h"
	extern void nsl_input_init();	
# endif

# ifdef NEURON_FG
	# include "nsl_all_neuron_include.h"
	extern void nsl_neuron_init();	
# endif

// global data definition

nsl_diff* NSL_DIFF;
nsl_interpreter* NSL_INTERPRETER;
nsl_system* NSL_SYSTEM;

nsl_interpreter* nsl_base::INTERPRETER = 0;
nsl_system* nsl_base::SYSTEM = 0;
nsl_public* nsl_base::SYS_INTERFACE = 0;
nsl_public* nsl_base::ASL_SYS = 0;
int BREAKFG = 0;
int INTERRUPTFG = 0;	// set to 1 if BREAKFG has been serviced 
			// left to the user for reset

# ifdef NSL_WIN	
extern void XNSLoutput(const char*);
extern void XNSLinit(int,char**);
extern void XNSLmain(int,char**);
extern void XNSLsysinit();
# endif

extern "C" 
{
	int NSLinit_interrupt_handler();
}

extern void nsl_kernel_init();	

extern void NSLetc();

// end addition

void NSLinit(int argc = 0,char** argv = NULL)
{
	cmd_out("NSL Version 3.0");
	cmd_out("(C) Copyright 1989-1997 Brain Simulation Laboratory All Rights Reserved");
	cmd_out("Center for Neural Engineering, University of Southern California");

# ifndef NSL_WIN	
	cmd_out("Initializing NSL Simulation System...");
# else
	cmd_out("Initializing XNSL Simulation System...");
# endif

// The order is VERY important
	nsl_sys_module* sys_mod;
	sys_mod = new nsl_sys_module("kernel",nsl_kernel_init,1);
# ifdef NSL_INPUT_FG
	sys_mod = new nsl_sys_module("input",nsl_input_init,1);
# endif
# ifdef NEURON_FG
	sys_mod = new nsl_sys_module("neuron",nsl_neuron_init,1);
# endif
	NSLetc();	// call any additional library instantiations
	
# ifdef NSL_WIN	
	XNSLsysinit();  // call graphics library instantiations
# endif
	
	if (NSL_SYSTEM == NULL)
	{
		NSL_SYSTEM = new nsl_system;
 	}

	if (NSL_SYSTEM == NULL)
	{
		cmd_error("Cannot create SYSTEM");
		exit(-1);
	}

	if (NSL_INTERPRETER == NULL)
	{
		NSL_INTERPRETER = new nsl_interpreter;
 	}

	if (NSL_INTERPRETER == NULL)
	{
		cmd_error("Cannot create NSL_INTERPRETER");
		exit(-1);
	}

// AW commented out 3/96
//	NSL_SYSTEM->set_argc_ptr(&argc);
//	NSL_SYSTEM->set_argv_ptr(argv);

// init XNSL before initiating all procedures
# ifdef NSL_WIN	
	XNSLinit(argc,argv);
# endif

# ifndef __GNUG__
//	cin.sync_with_stdio(); // sync stdio with C++ streams
# endif

	NSLinit_interrupt_handler();

// initiate
        NSL.add_shell(new nsl_cmd_buf); // creating new SHELL

	NSL.init(); // init all shells and LOG

	NSL_SYSTEM->init_proc(); // initiating SHELL (nsl_sys_module)

	if (NSL_DIFF == NULL)
	{
		NSL_DIFF = new nsl_diff;
 	}

	if (NSL_DIFF == NULL)
	{
		cmd_error("Cannot create NSL_DIFF");
		exit(-1);
	}

//	SchemaInit();
	
// execute nsl_system initializations
	NSL_SYSTEM->init(); 	// initiating MODELS (nsl_sys_module)
}

# ifdef NSL_WIN	
void NSLmain(int argc,char** argv)
# else
void NSLmain(int,char**)
# endif
{
# ifdef NSL_WIN	
	XNSLmain(argc,argv);
# else
	NSL.run(); // used if no windows
# endif

	cmd_out("Exiting NSL Simulation System");
}

// integrate similar ones

extern int XNSLoutput(const char* str,ostream&);

int NSLoutput(const char* str,ostream& out)
{
# ifdef NSL_WIN
	XNSLoutput(str,out);
# else
	out << str;
	out.flush();
# endif

	return 1;
}

int NSLoutput(const int str1,ostream& out)
{
	nsl_string str;

	sprintf(str,"%d",str1);
	NSLoutput(str,out);

	return 1;
}

int NSLoutput(const float str1,ostream& out)
{
	nsl_string str;

	sprintf(str,"%f",str1);
	NSLoutput(str,out);

	return 1;
}

int NSLoutput(const double str1,ostream& out)
{
	nsl_string str;

	sprintf(str,"%f",str1);
	NSLoutput(str,out);

	return 1;
}

int NSLoutput(const char* str)
{
# ifdef NSL_WIN
	XNSLoutput(str);
# else
	cout << str;
	cout.flush();
# endif

	return 1;
}

int NSLoutput(const int str1)
{
	nsl_string str;

	sprintf(str,"%d",str1);
	NSLoutput(str);

	return 1;
}

int NSLoutput(const float str1)
{
	nsl_string str;

	sprintf(str,"%f",str1);
	NSLoutput(str);

	return 1;
}

int NSLoutput(const double str1)
{
	nsl_string str;

	sprintf(str,"%f",str1);
	NSLoutput(str);

	return 1;
}

int NSLoutput(const char* str1,const char* str2)
{
	NSLoutput(str1);
	NSLoutput(str2);

	return 1;
}

