/* SCCS  @(#)nsl_input_matrix.C	1.1---95/08/13--20:55:05 */
//
//	nsl_input_matrix.c
//

# include "nsl_all_kernel_include.h"
# include "nsl_all_input_include.h"

nsl_input_matrix::nsl_input_matrix(const char* str,const int imx,
	const int jmx,int fg,int rfg) : nsl_matrix(str,imx,jmx,fg,rfg)
{
	set_type("input_matrix");

	layer_type = 1; // input layer
     	xz = 0;
	yz = 0;
	dx = 1.0;
	dy = 1.0;

	input_layer = new nsl_input_layer(str,this);
	m_children_list->append(input_layer);
}	
nsl_input_matrix::~nsl_input_matrix()
{
}

// input matrix assignment (input_matrix = matrix is NOT derived !!)

num_type** nsl_input_matrix::operator=(const nsl_input_matrix& a)
{
	if (a.get_imax() != imax || a.get_jmax() != jmax)
	{
		cmd_error("Error in nsl_input_matrix::operator=");
 		a.print_status(cerr);
		this->print_status(cerr);
		cmd_error("Inconsistent matrix sizes");
		return v;
	}

	for (int i = 0; i < imax; i++)
	   for (int j = 0; j < jmax; j++)
		v[i][j] = a.v[i][j];

	return v;
}
num_type** nsl_input_matrix::operator=(const nsl_matrix& a)
{
	if (a.get_imax() != imax || a.get_jmax() != jmax)
	{
		cmd_error("Error in nsl_input_matrix::operator=");
		a.print_status(cerr);
		this->print_status(cerr);
		cmd_error("Inconsistent matrix sizes");
		return v;
	}
	num_type** av = a.get_matrix();

	for (int i = 0; i < imax; i++)
	   for (int j = 0; j < jmax; j++)
		v[i][j] = av[i][j];

	return v;
}
num_type** nsl_input_matrix::operator=(const nsl_data& a)
{
	int imx = get_imax();  
	int jmx = get_jmax();

	num_type val = a.get_value();

	for (int i = 0; i < imx; i++)
	   for (int j = 0; j < jmx; j++)
		v[i][j] = val;

	return v;
}
num_type** nsl_input_matrix::operator=(const num_type a)
{
	int imx = get_imax();
	int jmx = get_jmax();

	for (int i = 0; i < imx; i++)
	   for (int j = 0; j < jmx; j++)
		v[i][j] = a;

	return v;
}
num_type** nsl_input_matrix::operator=(const int a)
{
	int imx = get_imax();
	int jmx = get_jmax();

	for (int i = 0; i < imx; i++)
	   for (int j = 0; j < jmx; j++)
		v[i][j] = (num_type) a;

	return v;
}

int nsl_input_matrix::run()
{
	if (input_layer != NULL)
		input_layer->run();
	else
		cmd_error("NULL input stim");

	return 1;
}

// print status

int nsl_input_matrix::print_status(ostream& out) const
{
	return print_input_matrix_status(out);
}
int nsl_input_matrix::print_input_matrix_status(ostream& out) const
{
 	nsl_matrix::print_status(out);

	if (input_layer != NULL)
		input_layer->print_status(out);
	else
		cmd_error("NULL input stim");

	return 1;
}
int nsl_input_matrix::print(ostream& out) const
{
	nsl_matrix::print(out);

  	out << "set input_layer xz " << xz << "\n";
	out << "set input_layer yz " << yz << "\n";
	out << "set input_layer dx " << dx << "\n";
	out << "set input_layer dy " << dy << "\n";
 
	out.flush();

	if (input_layer != NULL)
		input_layer->print(out);
	else
		cmd_error("NULL input stim");

	return 1;
}
int nsl_input_matrix::print(ostream& out,int i) const
{
	nsl_matrix::print(out);

  	out << "set input_layer xz " << xz << "\n";
	out << "set input_layer yz " << yz << "\n";
	out << "set input_layer dx " << dx << "\n";
	out << "set input_layer dy " << dy << "\n";
 
	out.flush();

	if (input_layer != NULL)
		input_layer->print(out,i);
	else
		cmd_error("NULL input stim");

	return 1;
}
