/* SCCS  @(#)nsl_system.C	1.1---95/10/13--16:37:16 */
//
//	nsl_system.C
//

# include "nsl_all_kernel_include.h"

nsl_system::nsl_system() : nsl_public("system")
{
	init_system();
}
nsl_system::~nsl_system()
{
	delete model_list;
	delete proc_list;
	delete file_type_list;
	delete diff_method_list;
	delete diff_model_list;
	delete thresh_lib_list;
	delete fm;
}

int nsl_system::init_system()
{
	set_base_type("system");
	set_name("system");
	m_parent = this;

  	SYS_TIME = 0.0;	
	reset_time();
	DELTA = 0.1;
	set_end_time((float)1.0);

	xsystem_fg = NSL_OFF;

 	model_total = 0;
	model_list = new nsl_list("model");

	proc_total = 0;
	proc_list = new nsl_list("proc");

	file_type_total = 0;
	file_type_list = new nsl_list("file_type");

	diff_method_total = 0;
	diff_method_list = new nsl_list("diff_method");

	diff_model_total = 0;
	diff_model_list = new nsl_list("diff_model");

	thresh_lib_total = 0;
	thresh_lib_list = new nsl_list("thresh_lib");

	fm = new nsl_file_manager;

//#ifndef __GNUG__
//	log.attach(1); // set log to standard output
//#endif

/*	if (NSL_DIFF == NULL)
	{
		NSL_DIFF = new nsl_diff;
		cmd_out("nsl_system:Initializing NSL_DIFF...");
 	}
*/
	DIFFM = new nsl_diff_manager; // initialize global diff manager
	DIFFM->set_m_parent(this);

	m_children_list->append(model_list);
	m_children_list->append(proc_list);
	m_children_list->append(file_type_list);
 	m_children_list->append(diff_method_list);
 	m_children_list->append(thresh_lib_list);
 	m_children_list->append(fm);
   	m_children_list->append(DIFFM);

	if (SYSTEM == NULL)
	{
		SYSTEM = this; // from nsl_root
		NSL_SYSTEM = this;
	}

	if (INTERPRETER == NULL)
	{
		INTERPRETER = new nsl_interpreter;
		NSL_INTERPRETER = INTERPRETER;
		cmd_out("nsl_system:Initializing NSL_INTERPRETER...");
	}

	char* env = getenv("NSL_DIR");
	if (env != NULL)
		strcpy(nsl_dir,env);
	else
	{
		cmd_error("NSL_DIR not set\nUse `setenv NSL_DIR *dir*'");
		strcpy(nsl_dir,".");
	}

	return 1;
}

// reset
void nsl_system::reset_time()
{
	CURRENT_TIME_STEP = 0;
	CURRENT_TIME = 0.0;
}
// delta
void nsl_system::set_delta(float i)
{
	DELTA = i;
}
// set
void nsl_system::set_time_step(int i)
{
	CURRENT_TIME_STEP = i;
	CURRENT_TIME = CURRENT_TIME_STEP*DELTA;
}
void nsl_system::set_time(int i) // called from 'run'
{
	CURRENT_TIME_STEP = i;
	CURRENT_TIME = CURRENT_TIME_STEP*DELTA;
}
void nsl_system::set_time(float i)
{
	CURRENT_TIME = i;
	CURRENT_TIME_STEP = (int) nsl_rint(CURRENT_TIME/DELTA);
}
// end
void nsl_system::set_end_time_step(int i)
{
	END_TIME_STEP = i;
	END_TIME = END_TIME_STEP*DELTA;
}
void nsl_system::set_end_time(float i)
{
	END_TIME = i;
	END_TIME_STEP = (int) nsl_rint(END_TIME/DELTA);
}
// increment
void nsl_system::incr_time()
{
	++CURRENT_TIME_STEP;
	CURRENT_TIME = CURRENT_TIME_STEP*DELTA;
}

// add model
 
int nsl_system::add_model(nsl_model* model)
{
	model->set_index(++model_total);
	model_list->append(model);
	model->set_m_parent(this);

	set_ptr(model); // OBJ = model
// AW98		set_model(model);

	return model_total;
}

nsl_base* get_hier_model(nsl_model* model,const char* fullstr)
{
	nsl_string mname;
	const char *str,*pname;
	nsl_base* ret_base = 0;

	if (model == NULL)
		return 0;

 	int mlen = strlen(fullstr); // full length

      	if ((str = (char *) strchr(fullstr,'.')) != NULL)
	{
		pname = str + 1; 	// path after first model name
		int plen = strlen(str); // extra path length
		strncpy(mname,fullstr,mlen-plen); // model name
		mname[mlen-plen]='\0';
	}
	else
	{
		strcpy(mname,fullstr);
		pname = 0;
	}

	nsl_list* model_list = model->get_model_list();

	if (model_list == NULL)
		return 0;

	// find model by name
	model = (nsl_model*) model_list->get_by_name(mname);

	// if model by name exists and pname is not null keep searching
	if (pname != NULL && model != NULL)
	{  
		if ((ret_base = get_hier_model(model,pname)) == NULL)
			ret_base = model->get_layer(pname);
	}
			
	return ret_base;
}

nsl_base* get_hier_object(const char* fullstr)
{
	nsl_string mname;
	const char *str,*pname;
	nsl_base* ret_base = 0;

 	int mlen = strlen(fullstr); // full length

      	if ((str = (char *) strchr(fullstr,'.')) != NULL)
	{
		pname = str + 1; 	// path after first model name
		int plen = strlen(str); // extra path length
		strncpy(mname,fullstr,mlen-plen); // model name
		mname[mlen-plen]='\0';
	}
	else
	{
		strcpy(mname,fullstr);
		pname = 0;
	}

	nsl_list* model_list = NSL_SYSTEM->get_model_list();

	if (model_list == NULL)
		return 0;


	// find model by name
	nsl_model* model = (nsl_model*) model_list->get_by_name(mname);

	// if model by name exists and pname is not null keep searching
	if (pname != NULL && model != NULL)
	{  
		if ((ret_base = get_hier_model(model,pname)) == NULL)
			ret_base = model->get_layer(pname);
	}
			
	return ret_base;
}

nsl_model* nsl_system::get_model(const char* fullstr)
{
	nsl_string mname;
	const char *str,*pname;

 	int mlen = strlen(fullstr); // full length

      	if ((str = (char *) strchr(fullstr,'.')) != NULL)
	{
		pname = str + 1; 	// path after first model name
		int plen = strlen(str); // extra path length
		strncpy(mname,fullstr,mlen-plen); // model name
		mname[mlen-plen]='\0';
	}
	else
	{
		strcpy(mname,fullstr);
		pname = 0;
	}

	nsl_model* model = (nsl_model*) model_list->get_by_name(mname);

	if (pname != NULL && model != NULL) 
		model = model->get_model(pname);

	return model;
}
nsl_model* nsl_system::get_model(const int mindex)
{
	return (nsl_model*) model_list->get_by_index(mindex);
}

int nsl_system::add_diff_method(nsl_diff_method* d)
{
	d->set_index(++diff_method_total);
	diff_method_list->append(d);
//	d->set_m_parent(this);

	return diff_method_total;
}

nsl_diff_method* nsl_system::get_diff_method(const char* mname)
{
	return (nsl_diff_method*) diff_method_list->get_by_name(mname);
}

int nsl_system::add_diff_model(nsl_diff* d)
{
	d->set_index(++diff_model_total);
	diff_model_list->append(d);
//	d->set_m_parent(this);

	return diff_model_total;
}

nsl_diff* nsl_system::get_diff_model(const char* mname)
{
	return (nsl_diff*) diff_model_list->get_by_name(mname);
}

int nsl_system::add_thresh_lib(nsl_thresh_library* d)
{
	d->set_index(++thresh_lib_total);
	thresh_lib_list->append(d);
//	d->set_m_parent(this);

	return thresh_lib_total;
}

nsl_thresh_library* nsl_system::get_thresh_lib(const char* mname)
{
	return (nsl_thresh_library*) thresh_lib_list->get_by_name(mname);
}

int nsl_system::init()
{
	nsl_model* model;

	model_list->wind_up();
	// initiating models (SYS_MODULE)
	while ((model = (nsl_model*) model_list->next()) != NULL)
		model->init();

	return 1;
}

// add proc
 
int nsl_system::insert_proc(nsl_module* m)
{
	proc_list->wind_up();
	proc_list->set_index(++proc_total);
	proc_list->insert(m);
	proc_list->set_m_parent(this);

 	return proc_total;
}
int nsl_system::add_proc(nsl_module* m)
{
	proc_list->set_index(++proc_total);
	proc_list->append(m);
	proc_list->set_m_parent(this);

 	return proc_total;
}
int nsl_system::init_proc()
{
 	if (proc_list != NULL)
	{
	    proc_list->wind_up();
	    nsl_module* m;
	    while (m = (nsl_module*) proc_list->next()) 
		((nsl_module*) m)->exec(); // global
	}
	else
	   cmd_out("NULL 'nsl_proc_list'");

	return 1;
}

// get model

nsl_module* nsl_system::get_proc(const char* mname)
{
	return (nsl_module*) proc_list->get_by_name(mname);
}
nsl_module* nsl_system::get_proc(const int mindex)
{
	return (nsl_module*) proc_list->get_by_index(mindex);
}

// i/o files

int nsl_system::add_file_type(nsl_file* sfile)
{
	sfile->set_index(++file_type_total);
	file_type_list->append(sfile);
	sfile->set_m_parent(this);

	return file_type_total;
}
nsl_file* nsl_system::get_file_type(const char* tname) // any dir
{
	const char* str;
	file_type_list->wind_up();
	nsl_file* sfile;
	while (sfile = (nsl_file*)file_type_list->next())
	{
		str = sfile->get_filename();	// w/o file ending
		if (strcmp(tname,str) == 0)
			return sfile;
		str = sfile->get_name(); 	// w/ file ending
		if (strcmp(tname,str) == 0)
			return sfile;
	}
	return (nsl_file*) 0;
}
nsl_file* nsl_system::get_file_type(const int tindex)
{
	return (nsl_file*) file_type_list->get_by_index(tindex);
}

// print

int nsl_system::print_system_status(ostream& out) const
{
//	print_base_status(out);
	
	out << "\n// model_total:\t\t" << model_total;
	out << "\n// diff_method_total:\t" << diff_method_total;
	out << "\n// thresh_lib_total:\t" << thresh_lib_total;
	out << "\n// file_type_total:\t" << file_type_total;
	out << "\n// nsl_dir:\t" << nsl_dir;
 	out.flush();

	return 1;
}
int nsl_system::print_status(ostream& out)  const // virtual
{
	print_system_status(out);
	return 1;
}
int nsl_system::print_system(ostream& out) const
{
//	print_base(out);

	print_system_status(out);
	
	return 1;
}
int nsl_system::print(ostream& out) const // virtual
{
	print_system(out);
	return 1;
}

// print all

int nsl_system::print_system_all_status(ostream& out) const
{
	print_system_status(out);
//	print_base_all_status(out);
	return 1;
}
int nsl_system::print_all_status(ostream& out)  const // virtual
{
	print_system_all_status(out);
	return 1;
}
int nsl_system::print_system_all(ostream& out) const
{
	print_system(out);
//	print_base_all(out);
	return 1;
}
int nsl_system::print_all(ostream& out) const // virtual
{
	print_system_all(out);
	return 1;
}

// read, write

int nsl_system::write_system(nsl_buffer&)
{
	return 1;
}
int nsl_system::write(nsl_buffer& buf)
{
	write_system(buf);
	return 1;
}
int nsl_system::read_system(nsl_buffer&)
{
	return 1;
}
int nsl_system::read(nsl_buffer& buf)
{
	read_system(buf);
	return 1;
}
